# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                     TraitedSpec, File, Directory, traits, isdefined,
                     InputMultiPath, OutputMultiPath)


class BRAINSCutInputSpec(CommandLineInputSpec):
    netConfiguration = File(
        desc=
        "XML File defining BRAINSCut parameters. OLD NAME. PLEASE USE modelConfigurationFilename instead.",
        exists=True,
        argstr="--netConfiguration %s")
    modelConfigurationFilename = File(
        desc="XML File defining BRAINSCut parameters",
        exists=True,
        argstr="--modelConfigurationFilename %s")
    trainModelStartIndex = traits.Int(
        desc="Starting iteration for training",
        argstr="--trainModelStartIndex %d")
    verbose = traits.Int(
        desc="print out some debugging information", argstr="--verbose %d")
    multiStructureThreshold = traits.Bool(
        desc="multiStructureThreshold module to deal with overlaping area",
        argstr="--multiStructureThreshold ")
    histogramEqualization = traits.Bool(
        desc=
        "A Histogram Equalization process could be added to the creating/applying process from Subject To Atlas. Default is false, which genreate input vectors without Histogram Equalization. ",
        argstr="--histogramEqualization ")
    computeSSEOn = traits.Bool(
        desc=
        "compute Sum of Square Error (SSE) along the trained model until the number of iteration given in the modelConfigurationFilename file",
        argstr="--computeSSEOn ")
    generateProbability = traits.Bool(
        desc="Generate probability map", argstr="--generateProbability ")
    createVectors = traits.Bool(
        desc="create vectors for training neural net",
        argstr="--createVectors ")
    trainModel = traits.Bool(
        desc="train the neural net", argstr="--trainModel ")
    NoTrainingVectorShuffling = traits.Bool(
        desc="If this flag is on, there will be no shuffling.",
        argstr="--NoTrainingVectorShuffling ")
    applyModel = traits.Bool(
        desc="apply the neural net", argstr="--applyModel ")
    validate = traits.Bool(
        desc=
        "validate data set.Just need for the first time run ( This is for validation of xml file and not working yet )",
        argstr="--validate ")
    method = traits.Enum("RandomForest", "ANN", argstr="--method %s")
    numberOfTrees = traits.Int(
        desc=
        " Random tree: number of trees. This is to be used when only one model with specified depth wish to be created. ",
        argstr="--numberOfTrees %d")
    randomTreeDepth = traits.Int(
        desc=
        " Random tree depth. This is to be used when only one model with specified depth wish to be created. ",
        argstr="--randomTreeDepth %d")
    modelFilename = traits.Str(
        desc=
        " model file name given from user (not by xml  configuration file) ",
        argstr="--modelFilename %s")


class BRAINSCutOutputSpec(TraitedSpec):
    pass


class BRAINSCut(SEMLikeCommandLine):
    """title: BRAINSCut (BRAINS)

category: Segmentation.Specialized

description: Automatic Segmentation using neural networks

version: 1.0

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: Vince Magnotta, Hans Johnson, Greg Harris, Kent Williams, Eunyoung Regina Kim

"""

    input_spec = BRAINSCutInputSpec
    output_spec = BRAINSCutOutputSpec
    _cmd = " BRAINSCut "
    _outputs_filenames = {}
    _redirect_x = False


class BRAINSROIAutoInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="The input image for finding the largest region filled mask.",
        exists=True,
        argstr="--inputVolume %s")
    outputROIMaskVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="The ROI automatically found from the input image.",
        argstr="--outputROIMaskVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "The inputVolume with optional [maskOutput|cropOutput] to the region of the brain mask.",
        argstr="--outputVolume %s")
    maskOutput = traits.Bool(
        desc="The inputVolume multiplied by the ROI mask.",
        argstr="--maskOutput ")
    cropOutput = traits.Bool(
        desc="The inputVolume cropped to the region of the ROI mask.",
        argstr="--cropOutput ")
    otsuPercentileThreshold = traits.Float(
        desc="Parameter to the Otsu threshold algorithm.",
        argstr="--otsuPercentileThreshold %f")
    thresholdCorrectionFactor = traits.Float(
        desc=
        "A factor to scale the Otsu algorithm's result threshold, in case clipping mangles the image.",
        argstr="--thresholdCorrectionFactor %f")
    closingSize = traits.Float(
        desc=
        "The Closing Size (in millimeters) for largest connected filled mask.  This value is divided by image spacing and rounded to the next largest voxel number.",
        argstr="--closingSize %f")
    ROIAutoDilateSize = traits.Float(
        desc=
        "This flag is only relavent when using ROIAUTO mode for initializing masks.  It defines the final dilation size to capture a bit of background outside the tissue region.  At setting of 10mm has been shown to help regularize a BSpline registration type so that there is some background constraints to match the edges of the head better.",
        argstr="--ROIAutoDilateSize %f")
    outputVolumePixelType = traits.Enum(
        "float",
        "short",
        "ushort",
        "int",
        "uint",
        "uchar",
        desc=
        "The output image Pixel Type is the scalar datatype for representation of the Output Volume.",
        argstr="--outputVolumePixelType %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class BRAINSROIAutoOutputSpec(TraitedSpec):
    outputROIMaskVolume = File(
        desc="The ROI automatically found from the input image.", exists=True)
    outputVolume = File(
        desc=
        "The inputVolume with optional [maskOutput|cropOutput] to the region of the brain mask.",
        exists=True)


class BRAINSROIAuto(SEMLikeCommandLine):
    """title: Foreground masking (BRAINS)

category: Segmentation.Specialized

description: This program is used to create a mask over the most prominant forground region in an image.  This is accomplished via a combination of otsu thresholding and a closing operation.  More documentation is available here: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/ForegroundMasking.

version: 2.4.1

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: Hans J. Johnson, hans-johnson -at- uiowa.edu, http://www.psychiatry.uiowa.edu

acknowledgements: Hans Johnson(1,3,4); Kent Williams(1); Gregory Harris(1), Vincent Magnotta(1,2,3);  Andriy Fedorov(5), fedorov -at- bwh.harvard.edu (Slicer integration); (1=University of Iowa Department of Psychiatry, 2=University of Iowa Department of Radiology, 3=University of Iowa Department of Biomedical Engineering, 4=University of Iowa Department of Electrical and Computer Engineering, 5=Surgical Planning Lab, Harvard)

"""

    input_spec = BRAINSROIAutoInputSpec
    output_spec = BRAINSROIAutoOutputSpec
    _cmd = " BRAINSROIAuto "
    _outputs_filenames = {
        'outputVolume': 'outputVolume.nii',
        'outputROIMaskVolume': 'outputROIMaskVolume.nii'
    }
    _redirect_x = False


class BRAINSConstellationDetectorInputSpec(CommandLineInputSpec):
    houghEyeDetectorMode = traits.Int(
        desc=
        ",                 This flag controls the mode of Hough eye detector.  By default, value of 1 is for T1W images, while the value of 0 is for T2W and PD images.,             ",
        argstr="--houghEyeDetectorMode %d")
    inputTemplateModel = File(
        desc="User-specified template model.,             ",
        exists=True,
        argstr="--inputTemplateModel %s")
    LLSModel = File(
        desc="Linear least squares model filename in HD5 format",
        exists=True,
        argstr="--LLSModel %s")
    inputVolume = File(
        desc="Input image in which to find ACPC points",
        exists=True,
        argstr="--inputVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "ACPC-aligned output image with the same voxels, but updated origin, and direction cosign so that the AC point would fall at the physical location (0.0,0.0,0.0), and the mid-sagital plane is the plane where physical L/R coordinate is 0.0.",
        argstr="--outputVolume %s")
    outputResampledVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "ACPC-aligned output image in a resampled unifor space.  Currently this is a 1mm, 256^3, Identity direction image.",
        argstr="--outputResampledVolume %s")
    outputTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "The filename for the original space to ACPC alignment to be written (in .h5 format).,             ",
        argstr="--outputTransform %s")
    outputLandmarksInInputSpace = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        ",               The filename for the new subject-specific landmark definition file in the same format produced by Slicer3 (.fcsv) with the landmarks in the original image space (the detected RP, AC, PC, and VN4) in it to be written.,             ",
        argstr="--outputLandmarksInInputSpace %s")
    outputLandmarksInACPCAlignedSpace = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        ",               The filename for the new subject-specific landmark definition file in the same format produced by Slicer3 (.fcsv) with the landmarks in the output image space (the detected RP, AC, PC, and VN4) in it to be written.,             ",
        argstr="--outputLandmarksInACPCAlignedSpace %s")
    outputMRML = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        ",               The filename for the new subject-specific scene definition file in the same format produced by Slicer3 (in .mrml format). Only the components that were specified by the user on command line would be generated. Compatible components include inputVolume, outputVolume, outputLandmarksInInputSpace, outputLandmarksInACPCAlignedSpace, and outputTransform.,             ",
        argstr="--outputMRML %s")
    outputVerificationScript = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        ",               The filename for the Slicer3 script that verifies the aligned landmarks against the aligned image file.  This will happen only in conjunction with saveOutputLandmarks and an outputVolume.,             ",
        argstr="--outputVerificationScript %s")
    mspQualityLevel = traits.Int(
        desc=
        ",                 Flag cotrols how agressive the MSP is estimated. 0=quick estimate (9 seconds), 1=normal estimate (11 seconds), 2=great estimate (22 seconds), 3=best estimate (58 seconds), NOTE: -1= Prealigned so no estimate!.,             ",
        argstr="--mspQualityLevel %d")
    otsuPercentileThreshold = traits.Float(
        desc=
        ",                 This is a parameter to FindLargestForegroundFilledMask, which is employed when acLowerBound is set and an outputUntransformedClippedVolume is requested.,             ",
        argstr="--otsuPercentileThreshold %f")
    acLowerBound = traits.Float(
        desc=
        ",                 When generating a resampled output image, replace the image with the BackgroundFillValue everywhere below the plane This Far in physical units (millimeters) below (inferior to) the AC point (as found by the model.)  The oversize default was chosen to have no effect.  Based on visualizing a thousand masks in the IPIG study, we recommend a limit no smaller than 80.0 mm.,             ",
        argstr="--acLowerBound %f")
    cutOutHeadInOutputVolume = traits.Bool(
        desc=
        ",                 Flag to cut out just the head tissue when producing an (un)transformed clipped volume.,             ",
        argstr="--cutOutHeadInOutputVolume ")
    outputUntransformedClippedVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Output image in which to store neck-clipped input image, with the use of --acLowerBound and maybe --cutOutHeadInUntransformedVolume.",
        argstr="--outputUntransformedClippedVolume %s")
    rescaleIntensities = traits.Bool(
        desc=
        ",                 Flag to turn on rescaling image intensities on input.,             ",
        argstr="--rescaleIntensities ")
    trimRescaledIntensities = traits.Float(
        desc=
        ",                 Turn on clipping the rescaled image one-tailed on input.  Units of standard deviations above the mean.  Very large values are very permissive.  Non-positive value turns clipping off.  Defaults to removing 0.00001 of a normal tail above the mean.,             ",
        argstr="--trimRescaledIntensities %f")
    rescaleIntensitiesOutputRange = InputMultiPath(
        traits.Int,
        desc=
        ",                 This pair of integers gives the lower and upper bounds on the signal portion of the output image.  Out-of-field voxels are taken from BackgroundFillValue.,             ",
        sep=",",
        argstr="--rescaleIntensitiesOutputRange %s")
    BackgroundFillValue = traits.Str(
        desc=
        "Fill the background of image with specified short int value. Enter number or use BIGNEG for a large negative number.",
        argstr="--BackgroundFillValue %s")
    interpolationMode = traits.Enum(
        "NearestNeighbor",
        "Linear",
        "ResampleInPlace",
        "BSpline",
        "WindowedSinc",
        "Hamming",
        "Cosine",
        "Welch",
        "Lanczos",
        "Blackman",
        desc=
        "Type of interpolation to be used when applying transform to moving volume.  Options are Linear, ResampleInPlace, NearestNeighbor, BSpline, or WindowedSinc",
        argstr="--interpolationMode %s")
    forceACPoint = InputMultiPath(
        traits.Float,
        desc=
        ",                 Use this flag to manually specify the AC point from the original image on the command line.,             ",
        sep=",",
        argstr="--forceACPoint %s")
    forcePCPoint = InputMultiPath(
        traits.Float,
        desc=
        ",                 Use this flag to manually specify the PC point from the original image on the command line.,             ",
        sep=",",
        argstr="--forcePCPoint %s")
    forceVN4Point = InputMultiPath(
        traits.Float,
        desc=
        ",                 Use this flag to manually specify the VN4 point from the original image on the command line.,             ",
        sep=",",
        argstr="--forceVN4Point %s")
    forceRPPoint = InputMultiPath(
        traits.Float,
        desc=
        ",                 Use this flag to manually specify the RP point from the original image on the command line.,             ",
        sep=",",
        argstr="--forceRPPoint %s")
    inputLandmarksEMSP = File(
        desc=
        ",               The filename for the new subject-specific landmark definition file in the same format produced by Slicer3 (in .fcsv) with the landmarks in the estimated MSP aligned space to be loaded. The detector will only process landmarks not enlisted on the file.,             ",
        exists=True,
        argstr="--inputLandmarksEMSP %s")
    forceHoughEyeDetectorReportFailure = traits.Bool(
        desc=
        ",                 Flag indicates whether the Hough eye detector should report failure,             ",
        argstr="--forceHoughEyeDetectorReportFailure ")
    rmpj = traits.Float(
        desc=
        ",               Search radius for MPJ in unit of mm,             ",
        argstr="--rmpj %f")
    rac = traits.Float(
        desc=",               Search radius for AC in unit of mm,             ",
        argstr="--rac %f")
    rpc = traits.Float(
        desc=",               Search radius for PC in unit of mm,             ",
        argstr="--rpc %f")
    rVN4 = traits.Float(
        desc=
        ",               Search radius for VN4 in unit of mm,             ",
        argstr="--rVN4 %f")
    debug = traits.Bool(
        desc=
        ",               Show internal debugging information.,             ",
        argstr="--debug ")
    verbose = traits.Bool(
        desc=",               Show more verbose output,             ",
        argstr="--verbose ")
    writeBranded2DImage = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        ",               The filename for the 2D .png branded midline debugging image.  This will happen only in conjunction with requesting an outputVolume.,             ",
        argstr="--writeBranded2DImage %s")
    resultsDir = traits.Either(
        traits.Bool,
        Directory(),
        hash_files=False,
        desc=
        ",               The directory for the debuging images to be written.,             ",
        argstr="--resultsDir %s")
    writedebuggingImagesLevel = traits.Int(
        desc=
        ",                 This flag controls if debugging images are produced.  By default value of 0 is no images.  Anything greater than zero will be increasing level of debugging images.,             ",
        argstr="--writedebuggingImagesLevel %d")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")
    atlasVolume = File(
        desc="Atlas volume image to be used for BRAINSFit registration",
        exists=True,
        argstr="--atlasVolume %s")
    atlasLandmarks = File(
        desc=
        "Atlas landmarks to be used for BRAINSFit registration initialization,             ",
        exists=True,
        argstr="--atlasLandmarks %s")
    atlasLandmarkWeights = File(
        desc=
        "Weights associated with atlas landmarks to be used for BRAINSFit registration initialization,             ",
        exists=True,
        argstr="--atlasLandmarkWeights %s")


class BRAINSConstellationDetectorOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "ACPC-aligned output image with the same voxels, but updated origin, and direction cosign so that the AC point would fall at the physical location (0.0,0.0,0.0), and the mid-sagital plane is the plane where physical L/R coordinate is 0.0.",
        exists=True)
    outputResampledVolume = File(
        desc=
        "ACPC-aligned output image in a resampled unifor space.  Currently this is a 1mm, 256^3, Identity direction image.",
        exists=True)
    outputTransform = File(
        desc=
        "The filename for the original space to ACPC alignment to be written (in .h5 format).,             ",
        exists=True)
    outputLandmarksInInputSpace = File(
        desc=
        ",               The filename for the new subject-specific landmark definition file in the same format produced by Slicer3 (.fcsv) with the landmarks in the original image space (the detected RP, AC, PC, and VN4) in it to be written.,             ",
        exists=True)
    outputLandmarksInACPCAlignedSpace = File(
        desc=
        ",               The filename for the new subject-specific landmark definition file in the same format produced by Slicer3 (.fcsv) with the landmarks in the output image space (the detected RP, AC, PC, and VN4) in it to be written.,             ",
        exists=True)
    outputMRML = File(
        desc=
        ",               The filename for the new subject-specific scene definition file in the same format produced by Slicer3 (in .mrml format). Only the components that were specified by the user on command line would be generated. Compatible components include inputVolume, outputVolume, outputLandmarksInInputSpace, outputLandmarksInACPCAlignedSpace, and outputTransform.,             ",
        exists=True)
    outputVerificationScript = File(
        desc=
        ",               The filename for the Slicer3 script that verifies the aligned landmarks against the aligned image file.  This will happen only in conjunction with saveOutputLandmarks and an outputVolume.,             ",
        exists=True)
    outputUntransformedClippedVolume = File(
        desc=
        "Output image in which to store neck-clipped input image, with the use of --acLowerBound and maybe --cutOutHeadInUntransformedVolume.",
        exists=True)
    writeBranded2DImage = File(
        desc=
        ",               The filename for the 2D .png branded midline debugging image.  This will happen only in conjunction with requesting an outputVolume.,             ",
        exists=True)
    resultsDir = Directory(
        desc=
        ",               The directory for the debuging images to be written.,             ",
        exists=True)


class BRAINSConstellationDetector(SEMLikeCommandLine):
    """title: Brain Landmark Constellation Detector (BRAINS)

category: Segmentation.Specialized

description: This program will find the mid-sagittal plane, a constellation of landmarks in a volume, and create an AC/PC aligned data set with the AC point at the center of the voxel lattice (labeled at the origin of the image physical space.)  Part of this work is an extention of the algorithms originally described by Dr. Babak A. Ardekani, Alvin H. Bachman, Model-based automatic detection of the anterior and posterior commissures on MRI scans, NeuroImage, Volume 46, Issue 3, 1 July 2009, Pages 677-682, ISSN 1053-8119, DOI: 10.1016/j.neuroimage.2009.02.030.  (http://www.sciencedirect.com/science/article/B6WNP-4VRP25C-4/2/8207b962a38aa83c822c6379bc43fe4c)

version: 1.0

documentation-url: http://www.nitrc.org/projects/brainscdetector/

"""

    input_spec = BRAINSConstellationDetectorInputSpec
    output_spec = BRAINSConstellationDetectorOutputSpec
    _cmd = " BRAINSConstellationDetector "
    _outputs_filenames = {
        'outputVolume':
        'outputVolume.nii.gz',
        'outputMRML':
        'outputMRML.mrml',
        'resultsDir':
        'resultsDir',
        'outputResampledVolume':
        'outputResampledVolume.nii.gz',
        'outputTransform':
        'outputTransform.h5',
        'writeBranded2DImage':
        'writeBranded2DImage.png',
        'outputLandmarksInACPCAlignedSpace':
        'outputLandmarksInACPCAlignedSpace.fcsv',
        'outputLandmarksInInputSpace':
        'outputLandmarksInInputSpace.fcsv',
        'outputUntransformedClippedVolume':
        'outputUntransformedClippedVolume.nii.gz',
        'outputVerificationScript':
        'outputVerificationScript.sh'
    }
    _redirect_x = False


class BRAINSCreateLabelMapFromProbabilityMapsInputSpec(CommandLineInputSpec):
    inputProbabilityVolume = InputMultiPath(
        File(exists=True),
        desc="The list of proobabilityimages.",
        argstr="--inputProbabilityVolume %s...")
    priorLabelCodes = InputMultiPath(
        traits.Int,
        desc=
        "A list of PriorLabelCode values used for coding the output label images",
        sep=",",
        argstr="--priorLabelCodes %s")
    foregroundPriors = InputMultiPath(
        traits.Int,
        desc="A list: For each Prior Label, 1 if foreground, 0 if background",
        sep=",",
        argstr="--foregroundPriors %s")
    nonAirRegionMask = File(
        desc=
        "a mask representing the \'NonAirRegion\' -- Just force pixels in this region to zero",
        exists=True,
        argstr="--nonAirRegionMask %s")
    inclusionThreshold = traits.Float(
        desc="tolerance for inclusion", argstr="--inclusionThreshold %f")
    dirtyLabelVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="the labels prior to cleaning",
        argstr="--dirtyLabelVolume %s")
    cleanLabelVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="the foreground labels volume",
        argstr="--cleanLabelVolume %s")


class BRAINSCreateLabelMapFromProbabilityMapsOutputSpec(TraitedSpec):
    dirtyLabelVolume = File(desc="the labels prior to cleaning", exists=True)
    cleanLabelVolume = File(desc="the foreground labels volume", exists=True)


class BRAINSCreateLabelMapFromProbabilityMaps(SEMLikeCommandLine):
    """title: Create Label Map From Probability Maps (BRAINS)

category: Segmentation.Specialized

description: Given A list of Probability Maps, generate a LabelMap.

"""

    input_spec = BRAINSCreateLabelMapFromProbabilityMapsInputSpec
    output_spec = BRAINSCreateLabelMapFromProbabilityMapsOutputSpec
    _cmd = " BRAINSCreateLabelMapFromProbabilityMaps "
    _outputs_filenames = {
        'dirtyLabelVolume': 'dirtyLabelVolume.nii',
        'cleanLabelVolume': 'cleanLabelVolume.nii'
    }
    _redirect_x = False


class BinaryMaskEditorBasedOnLandmarksInputSpec(CommandLineInputSpec):
    inputBinaryVolume = File(
        desc="Input binary image in which to be edited",
        exists=True,
        argstr="--inputBinaryVolume %s")
    outputBinaryVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output binary image in which to be edited",
        argstr="--outputBinaryVolume %s")
    inputLandmarksFilename = File(
        desc=
        " The filename for the  landmark definition file in the same format produced by Slicer3 (.fcsv). ",
        exists=True,
        argstr="--inputLandmarksFilename %s")
    inputLandmarkNames = InputMultiPath(
        traits.Str,
        desc=
        " A target input landmark name to be edited. This should be listed in the inputLandmakrFilename Given. ",
        sep=",",
        argstr="--inputLandmarkNames %s")
    setCutDirectionForLandmark = InputMultiPath(
        traits.Str,
        desc=
        "Setting the cutting out direction of the input binary image to the one of anterior, posterior, left, right, superior or posterior. (ENUMERATION: ANTERIOR, POSTERIOR, LEFT, RIGHT, SUPERIOR, POSTERIOR) ",
        sep=",",
        argstr="--setCutDirectionForLandmark %s")
    setCutDirectionForObliquePlane = InputMultiPath(
        traits.Str,
        desc=
        "If this is true, the mask will be thresholded out to the direction of inferior, posterior,  and/or left. Default behavrior is that cutting out to the direction of superior, anterior and/or right. ",
        sep=",",
        argstr="--setCutDirectionForObliquePlane %s")
    inputLandmarkNamesForObliquePlane = InputMultiPath(
        traits.Str,
        desc=
        " Three subset landmark names of inputLandmarksFilename for a oblique plane computation. The plane computed for binary volume editing. ",
        sep=",",
        argstr="--inputLandmarkNamesForObliquePlane %s")


class BinaryMaskEditorBasedOnLandmarksOutputSpec(TraitedSpec):
    outputBinaryVolume = File(
        desc="Output binary image in which to be edited", exists=True)


class BinaryMaskEditorBasedOnLandmarks(SEMLikeCommandLine):
    """title: BRAINS Binary Mask Editor Based On Landmarks(BRAINS)

category: Segmentation.Specialized

version: 1.0

documentation-url: http://www.nitrc.org/projects/brainscdetector/

"""

    input_spec = BinaryMaskEditorBasedOnLandmarksInputSpec
    output_spec = BinaryMaskEditorBasedOnLandmarksOutputSpec
    _cmd = " BinaryMaskEditorBasedOnLandmarks "
    _outputs_filenames = {'outputBinaryVolume': 'outputBinaryVolume.nii'}
    _redirect_x = False


class BRAINSMultiSTAPLEInputSpec(CommandLineInputSpec):
    inputCompositeT1Volume = File(
        desc=
        "Composite T1, all label maps transofrmed into the space for this image.",
        exists=True,
        argstr="--inputCompositeT1Volume %s")
    inputLabelVolume = InputMultiPath(
        File(exists=True),
        desc="The list of proobabilityimages.",
        argstr="--inputLabelVolume %s...")
    inputTransform = InputMultiPath(
        File(exists=True),
        desc="transforms to apply to label volumes",
        argstr="--inputTransform %s...")
    labelForUndecidedPixels = traits.Int(
        desc="Label for undecided pixels",
        argstr="--labelForUndecidedPixels %d")
    resampledVolumePrefix = traits.Str(
        desc="if given, write out resampled volumes with this prefix",
        argstr="--resampledVolumePrefix %s")
    skipResampling = traits.Bool(
        desc="Omit resampling images into reference space",
        argstr="--skipResampling ")
    outputMultiSTAPLE = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="the MultiSTAPLE average of input label volumes",
        argstr="--outputMultiSTAPLE %s")
    outputConfusionMatrix = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Confusion Matrix",
        argstr="--outputConfusionMatrix %s")


class BRAINSMultiSTAPLEOutputSpec(TraitedSpec):
    outputMultiSTAPLE = File(
        desc="the MultiSTAPLE average of input label volumes", exists=True)
    outputConfusionMatrix = File(desc="Confusion Matrix", exists=True)


class BRAINSMultiSTAPLE(SEMLikeCommandLine):
    """title: Create best representative label map)

category: Segmentation.Specialized

description: given a list of label map images, create a representative/average label map.

"""

    input_spec = BRAINSMultiSTAPLEInputSpec
    output_spec = BRAINSMultiSTAPLEOutputSpec
    _cmd = " BRAINSMultiSTAPLE "
    _outputs_filenames = {
        'outputMultiSTAPLE': 'outputMultiSTAPLE.nii',
        'outputConfusionMatrix': 'outputConfusionMatrixh5|mat|txt'
    }
    _redirect_x = False


class BRAINSABCInputSpec(CommandLineInputSpec):
    inputVolumes = InputMultiPath(
        File(exists=True),
        desc="The list of input image files to be segmented.",
        argstr="--inputVolumes %s...")
    atlasDefinition = File(
        desc="Contains all parameters for Atlas",
        exists=True,
        argstr="--atlasDefinition %s")
    restoreState = File(
        desc="The initial state for the registration process",
        exists=True,
        argstr="--restoreState %s")
    saveState = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "(optional) Filename to which save the final state of the registration",
        argstr="--saveState %s")
    inputVolumeTypes = InputMultiPath(
        traits.Str,
        desc="The list of input image types corresponding to the inputVolumes.",
        sep=",",
        argstr="--inputVolumeTypes %s")
    outputDir = traits.Either(
        traits.Bool,
        Directory(),
        hash_files=False,
        desc="Ouput directory",
        argstr="--outputDir %s")
    atlasToSubjectTransformType = traits.Enum(
        "Identity",
        "Rigid",
        "Affine",
        "BSpline",
        "SyN",
        desc=
        " What type of linear transform type do you want to use to register the atlas to the reference subject image.",
        argstr="--atlasToSubjectTransformType %s")
    atlasToSubjectTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="The transform from atlas to the subject",
        argstr="--atlasToSubjectTransform %s")
    atlasToSubjectInitialTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="The initial transform from atlas to the subject",
        argstr="--atlasToSubjectInitialTransform %s")
    subjectIntermodeTransformType = traits.Enum(
        "Identity",
        "Rigid",
        "Affine",
        "BSpline",
        desc=
        " What type of linear transform type do you want to use to register the atlas to the reference subject image.",
        argstr="--subjectIntermodeTransformType %s")
    outputVolumes = traits.Either(
        traits.Bool,
        InputMultiPath(File(), ),
        hash_files=False,
        desc=
        "Corrected Output Images: should specify the same number of images as inputVolume, if only one element is given, then it is used as a file pattern where %s is replaced by the imageVolumeType, and %d by the index list location.",
        argstr="--outputVolumes %s...")
    outputLabels = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output Label Image",
        argstr="--outputLabels %s")
    outputDirtyLabels = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output Dirty Label Image",
        argstr="--outputDirtyLabels %s")
    posteriorTemplate = traits.Str(
        desc="filename template for Posterior output files",
        argstr="--posteriorTemplate %s")
    outputFormat = traits.Enum(
        "NIFTI",
        "Meta",
        "Nrrd",
        desc="Output format",
        argstr="--outputFormat %s")
    interpolationMode = traits.Enum(
        "BSpline",
        "NearestNeighbor",
        "WindowedSinc",
        "Linear",
        "ResampleInPlace",
        "Hamming",
        "Cosine",
        "Welch",
        "Lanczos",
        "Blackman",
        desc=
        "Type of interpolation to be used when applying transform to moving volume.  Options are Linear, NearestNeighbor, BSpline, WindowedSinc, or ResampleInPlace.  The ResampleInPlace option will create an image with the same discrete voxel values and will adjust the origin and direction of the physical space interpretation.",
        argstr="--interpolationMode %s")
    maxIterations = traits.Int(
        desc="Filter iterations", argstr="--maxIterations %d")
    medianFilterSize = InputMultiPath(
        traits.Int,
        desc=
        "The radius for the optional MedianImageFilter preprocessing in all 3 directions.",
        sep=",",
        argstr="--medianFilterSize %s")
    filterIteration = traits.Int(
        desc="Filter iterations", argstr="--filterIteration %d")
    filterTimeStep = traits.Float(
        desc=
        "Filter time step should be less than (PixelSpacing/(1^(DIM+1)), value is set to negative, then allow automatic setting of this value. ",
        argstr="--filterTimeStep %f")
    filterMethod = traits.Enum(
        "None",
        "CurvatureFlow",
        "GradientAnisotropicDiffusion",
        "Median",
        desc="Filter method for preprocessing of registration",
        argstr="--filterMethod %s")
    maxBiasDegree = traits.Int(
        desc="Maximum bias degree", argstr="--maxBiasDegree %d")
    useKNN = traits.Bool(
        desc="Use the KNN stage of estimating posteriors.", argstr="--useKNN ")
    purePlugsThreshold = traits.Float(
        desc=
        "If this threshold value is greater than zero, only pure samples are used to compute the distributions in EM classification, and only pure samples are used for KNN training. The default value is set to 0, that means not using pure plugs. However, a value of 0.2 is suggested if you want to activate using pure plugs option.",
        argstr="--purePlugsThreshold %f")
    numberOfSubSamplesInEachPlugArea = InputMultiPath(
        traits.Int,
        desc=
        "Number of continous index samples taken at each direction of lattice space for each plug volume.",
        sep=",",
        argstr="--numberOfSubSamplesInEachPlugArea %s")
    atlasWarpingOff = traits.Bool(
        desc="Deformable registration of atlas to subject",
        argstr="--atlasWarpingOff ")
    gridSize = InputMultiPath(
        traits.Int,
        desc="Grid size for atlas warping with BSplines",
        sep=",",
        argstr="--gridSize %s")
    defaultSuffix = traits.Str(argstr="--defaultSuffix %s")
    implicitOutputs = traits.Either(
        traits.Bool,
        InputMultiPath(File(), ),
        hash_files=False,
        desc=
        "Outputs to be made available to NiPype. Needed because not all BRAINSABC outputs have command line arguments.",
        argstr="--implicitOutputs %s...")
    debuglevel = traits.Int(
        desc=
        "Display debug messages, and produce debug intermediate results.  0=OFF, 1=Minimal, 10=Maximum debugging.",
        argstr="--debuglevel %d")
    writeLess = traits.Bool(
        desc="Does not write posteriors and filtered, bias corrected images",
        argstr="--writeLess ")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class BRAINSABCOutputSpec(TraitedSpec):
    saveState = File(
        desc=
        "(optional) Filename to which save the final state of the registration",
        exists=True)
    outputDir = Directory(desc="Ouput directory", exists=True)
    atlasToSubjectTransform = File(
        desc="The transform from atlas to the subject", exists=True)
    atlasToSubjectInitialTransform = File(
        desc="The initial transform from atlas to the subject", exists=True)
    outputVolumes = OutputMultiPath(
        File(exists=True),
        desc=
        "Corrected Output Images: should specify the same number of images as inputVolume, if only one element is given, then it is used as a file pattern where %s is replaced by the imageVolumeType, and %d by the index list location."
    )
    outputLabels = File(desc="Output Label Image", exists=True)
    outputDirtyLabels = File(desc="Output Dirty Label Image", exists=True)
    implicitOutputs = OutputMultiPath(
        File(exists=True),
        desc=
        "Outputs to be made available to NiPype. Needed because not all BRAINSABC outputs have command line arguments."
    )


class BRAINSABC(SEMLikeCommandLine):
    """title: Intra-subject registration, bias Correction, and tissue classification (BRAINS)

category: Segmentation.Specialized

description: Atlas-based tissue segmentation method.  This is an algorithmic extension of work done by XXXX at UNC and Utah XXXX need more description here.

"""

    input_spec = BRAINSABCInputSpec
    output_spec = BRAINSABCOutputSpec
    _cmd = " BRAINSABC "
    _outputs_filenames = {
        'saveState': 'saveState.h5',
        'outputLabels': 'outputLabels.nii.gz',
        'atlasToSubjectTransform': 'atlasToSubjectTransform.h5',
        'atlasToSubjectInitialTransform': 'atlasToSubjectInitialTransform.h5',
        'outputDirtyLabels': 'outputDirtyLabels.nii.gz',
        'outputVolumes': 'outputVolumes.nii.gz',
        'outputDir': 'outputDir',
        'implicitOutputs': 'implicitOutputs.nii.gz'
    }
    _redirect_x = False


class ESLRInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Input Label Volume", exists=True, argstr="--inputVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output Label Volume",
        argstr="--outputVolume %s")
    low = traits.Int(
        desc="The lower bound of the labels to be used.", argstr="--low %d")
    high = traits.Int(
        desc="The higher bound of the labels to be used.", argstr="--high %d")
    closingSize = traits.Int(
        desc="The closing size for hole filling.", argstr="--closingSize %d")
    openingSize = traits.Int(
        desc="The opening size for hole filling.", argstr="--openingSize %d")
    safetySize = traits.Int(
        desc="The safetySize size for the clipping region.",
        argstr="--safetySize %d")
    preserveOutside = traits.Bool(
        desc="For values outside the specified range, preserve those values.",
        argstr="--preserveOutside ")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class ESLROutputSpec(TraitedSpec):
    outputVolume = File(desc="Output Label Volume", exists=True)


class ESLR(SEMLikeCommandLine):
    """title: Clean Contiguous Label Map (BRAINS)

category: Segmentation.Specialized

description: From a range of label map values, extract the largest contiguous region of those labels

"""

    input_spec = ESLRInputSpec
    output_spec = ESLROutputSpec
    _cmd = " ESLR "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii.gz'}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..epi import EpiReg


def test_EpiReg_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        echospacing=dict(argstr='--echospacing=%f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi=dict(
            argstr='--epi=%s',
            extensions=None,
            mandatory=True,
            position=-4,
        ),
        fmap=dict(
            argstr='--fmap=%s',
            extensions=None,
        ),
        fmapmag=dict(
            argstr='--fmapmag=%s',
            extensions=None,
        ),
        fmapmagbrain=dict(
            argstr='--fmapmagbrain=%s',
            extensions=None,
        ),
        no_clean=dict(
            argstr='--noclean',
            usedefault=True,
        ),
        no_fmapreg=dict(argstr='--nofmapreg', ),
        out_base=dict(
            argstr='--out=%s',
            position=-1,
            usedefault=True,
        ),
        output_type=dict(),
        pedir=dict(argstr='--pedir=%s', ),
        t1_brain=dict(
            argstr='--t1brain=%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        t1_head=dict(
            argstr='--t1=%s',
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        weight_image=dict(
            argstr='--weight=%s',
            extensions=None,
        ),
        wmseg=dict(
            argstr='--wmseg=%s',
            extensions=None,
        ),
    )
    inputs = EpiReg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_EpiReg_outputs():
    output_map = dict(
        epi2str_inv=dict(extensions=None, ),
        epi2str_mat=dict(extensions=None, ),
        fmap2epi_mat=dict(extensions=None, ),
        fmap2str_mat=dict(extensions=None, ),
        fmap_epi=dict(extensions=None, ),
        fmap_str=dict(extensions=None, ),
        fmapmag_str=dict(extensions=None, ),
        fullwarp=dict(extensions=None, ),
        out_1vol=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
        seg=dict(extensions=None, ),
        shiftmap=dict(extensions=None, ),
        wmedge=dict(extensions=None, ),
        wmseg=dict(extensions=None, ),
    )
    outputs = EpiReg.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from .tracking import (Tracks2Prob, FilterTracks, StreamlineTrack,
                       DiffusionTensorStreamlineTrack,
                       SphericallyDeconvolutedStreamlineTrack,
                       ProbabilisticSphericallyDeconvolutedStreamlineTrack)
from .tensors import (FSL2MRTrix, ConstrainedSphericalDeconvolution,
                      DWI2SphericalHarmonicsImage, EstimateResponseForSH,
                      GenerateDirections, FindShPeaks, Directions2Amplitude)
from .preprocess import (MRConvert, MRMultiply, MRTrixViewer, MRTrixInfo,
                         GenerateWhiteMatterMask, DWI2Tensor,
                         Tensor2ApparentDiffusion, Tensor2FractionalAnisotropy,
                         Tensor2Vector, MedianFilter3D, Erode, Threshold)
from .convert import MRTrix2TrackVis



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import Bfc


def test_Bfc_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        biasEstimateConvergenceThreshold=dict(argstr='--beps %f', ),
        biasEstimateSpacing=dict(argstr='-s %d', ),
        biasFieldEstimatesOutputPrefix=dict(argstr='--biasprefix %s', ),
        biasRange=dict(argstr='%s', ),
        controlPointSpacing=dict(argstr='-c %d', ),
        convergenceThreshold=dict(argstr='--eps %f', ),
        correctWholeVolume=dict(argstr='--extrapolate', ),
        correctedImagesOutputPrefix=dict(argstr='--prefix %s', ),
        correctionScheduleFile=dict(
            argstr='--schedule %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        histogramRadius=dict(argstr='-r %d', ),
        histogramType=dict(argstr='%s', ),
        inputMRIFile=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        inputMaskFile=dict(
            argstr='-m %s',
            extensions=None,
            hash_files=False,
        ),
        intermediate_file_type=dict(argstr='%s', ),
        iterativeMode=dict(argstr='--iterate', ),
        maxBias=dict(
            argstr='-U %f',
            usedefault=True,
        ),
        minBias=dict(
            argstr='-L %f',
            usedefault=True,
        ),
        outputBiasField=dict(
            argstr='--bias %s',
            extensions=None,
            hash_files=False,
        ),
        outputMRIVolume=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        outputMaskedBiasField=dict(
            argstr='--maskedbias %s',
            extensions=None,
            hash_files=False,
        ),
        splineLambda=dict(argstr='-w %f', ),
        timer=dict(argstr='--timer', ),
        verbosityLevel=dict(argstr='-v %d', ),
    )
    inputs = Bfc.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Bfc_outputs():
    output_map = dict(
        correctionScheduleFile=dict(extensions=None, ),
        outputBiasField=dict(extensions=None, ),
        outputMRIVolume=dict(extensions=None, ),
        outputMaskedBiasField=dict(extensions=None, ),
    )
    outputs = Bfc.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..thresholdscalarvolume import ThresholdScalarVolume


def test_ThresholdScalarVolume_inputs():
    input_map = dict(
        InputVolume=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        OutputVolume=dict(
            argstr='%s',
            hash_files=False,
            position=-1,
        ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        lower=dict(argstr='--lower %d', ),
        outsidevalue=dict(argstr='--outsidevalue %d', ),
        threshold=dict(argstr='--threshold %d', ),
        thresholdtype=dict(argstr='--thresholdtype %s', ),
        upper=dict(argstr='--upper %d', ),
    )
    inputs = ThresholdScalarVolume.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ThresholdScalarVolume_outputs():
    output_map = dict(OutputVolume=dict(
        extensions=None,
        position=-1,
    ), )
    outputs = ThresholdScalarVolume.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, absolute_import,
                        unicode_literals)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Notes


def test_Notes_inputs():
    input_map = dict(
        add=dict(argstr='-a "%s"', ),
        add_history=dict(
            argstr='-h "%s"',
            xor=['rep_history'],
        ),
        args=dict(argstr='%s', ),
        delete=dict(argstr='-d %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
        ),
        outputtype=dict(),
        rep_history=dict(
            argstr='-HH "%s"',
            xor=['add_history'],
        ),
        ses=dict(argstr='-ses', ),
    )
    inputs = Notes.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Notes_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Notes.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..io import XNATSink


def test_XNATSink_inputs():
    input_map = dict(
        _outputs=dict(usedefault=True, ),
        assessor_id=dict(xor=['reconstruction_id'], ),
        cache_dir=dict(),
        config=dict(
            extensions=None,
            mandatory=True,
            xor=['server'],
        ),
        experiment_id=dict(mandatory=True, ),
        project_id=dict(mandatory=True, ),
        pwd=dict(),
        reconstruction_id=dict(xor=['assessor_id'], ),
        server=dict(
            mandatory=True,
            requires=['user', 'pwd'],
            xor=['config'],
        ),
        share=dict(usedefault=True, ),
        subject_id=dict(mandatory=True, ),
        user=dict(),
    )
    inputs = XNATSink.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Additional handy utilities for testing
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import range, object, open

import os
import time
import shutil
import signal
import subprocess
from subprocess import CalledProcessError
from tempfile import mkdtemp
from future.utils import raise_from
from ..utils.misc import package_check

__docformat__ = 'restructuredtext'

import numpy as np
import nibabel as nb


class TempFATFS(object):
    def __init__(self, size_in_mbytes=8, delay=0.5):
        """Temporary filesystem for testing non-POSIX filesystems on a POSIX
        system.

        with TempFATFS() as fatdir:
            target = os.path.join(fatdir, 'target')
            copyfile(file1, target, copy=False)
            assert not os.path.islink(target)

        Arguments
        ---------
        size_in_mbytes : int
            Size (in MiB) of filesystem to create
        delay : float
            Time (in seconds) to wait for fusefat to start, stop
        """
        self.delay = delay
        self.tmpdir = mkdtemp()
        self.dev_null = open(os.devnull, 'wb')

        vfatfile = os.path.join(self.tmpdir, 'vfatblock')
        self.vfatmount = os.path.join(self.tmpdir, 'vfatmount')
        self.canary = os.path.join(self.vfatmount, '.canary')

        with open(vfatfile, 'wb') as fobj:
            fobj.write(b'\x00' * (int(size_in_mbytes) << 20))
        os.mkdir(self.vfatmount)

        mkfs_args = ['mkfs.vfat', vfatfile]
        mount_args = ['fusefat', '-o', 'rw+', '-f', vfatfile, self.vfatmount]

        try:
            subprocess.check_call(
                args=mkfs_args, stdout=self.dev_null, stderr=self.dev_null)
        except CalledProcessError as e:
            raise_from(IOError("mkfs.vfat failed"), e)

        try:
            self.fusefat = subprocess.Popen(
                args=mount_args, stdout=self.dev_null, stderr=self.dev_null)
        except OSError as e:
            raise_from(IOError("fusefat is not installed"), e)

        time.sleep(self.delay)

        if self.fusefat.poll() is not None:
            raise IOError("fusefat terminated too soon")

        open(self.canary, 'wb').close()

    def __enter__(self):
        return self.vfatmount

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.fusefat is not None:
            self.fusefat.send_signal(signal.SIGINT)

            # Allow 1s to return without sending terminate
            for count in range(10):
                time.sleep(0.1)
                if self.fusefat.poll() is not None:
                    break
            else:
                self.fusefat.terminate()
            time.sleep(self.delay)
            assert not os.path.exists(self.canary)
        self.dev_null.close()
        shutil.rmtree(self.tmpdir)


def save_toy_nii(ndarray, filename):
    toy = nb.Nifti1Image(ndarray, np.eye(4))
    nb.nifti1.save(toy, filename)
    return filename



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Means


def test_Means_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        count=dict(argstr='-count', ),
        datum=dict(argstr='-datum %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file_a=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        in_file_b=dict(
            argstr='%s',
            extensions=None,
            position=-1,
        ),
        mask_inter=dict(argstr='-mask_inter', ),
        mask_union=dict(argstr='-mask_union', ),
        non_zero=dict(argstr='-non_zero', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source='in_file_a',
            name_template='%s_mean',
        ),
        outputtype=dict(),
        scale=dict(argstr='-%sscale', ),
        sqr=dict(argstr='-sqr', ),
        std_dev=dict(argstr='-stdev', ),
        summ=dict(argstr='-sum', ),
    )
    inputs = Means.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Means_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Means.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import Label2Annot


def test_Label2Annot_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        color_table=dict(
            argstr='--ctab %s',
            extensions=None,
        ),
        copy_inputs=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hemisphere=dict(
            argstr='--hemi %s',
            mandatory=True,
        ),
        in_labels=dict(
            argstr='--l %s...',
            mandatory=True,
        ),
        keep_max=dict(argstr='--maxstatwinner', ),
        orig=dict(
            extensions=None,
            mandatory=True,
        ),
        out_annot=dict(
            argstr='--a %s',
            mandatory=True,
        ),
        subject_id=dict(
            argstr='--s %s',
            mandatory=True,
            usedefault=True,
        ),
        subjects_dir=dict(),
        verbose_off=dict(argstr='--noverbose', ),
    )
    inputs = Label2Annot.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Label2Annot_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Label2Annot.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
'''
Algorithms to compute statistics on :abbr:`fMRI (functional MRI)`
'''
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
import os

import numpy as np
import nibabel as nb

from ..interfaces.base import (traits, TraitedSpec, LibraryBaseInterface,
                               SimpleInterface, BaseInterfaceInputSpec, File,
                               InputMultiPath)


class NilearnBaseInterface(LibraryBaseInterface):
    _pkg = 'nilearn'


class SignalExtractionInputSpec(BaseInterfaceInputSpec):
    in_file = File(exists=True, mandatory=True, desc='4-D fMRI nii file')
    label_files = InputMultiPath(
        File(exists=True),
        mandatory=True,
        desc='a 3-D label image, with 0 denoting '
        'background, or a list of 3-D probability '
        'maps (one per label) or the equivalent 4D '
        'file.')
    class_labels = traits.List(
        mandatory=True,
        desc='Human-readable labels for each segment '
        'in the label file, in order. The length of '
        'class_labels must be equal to the number of '
        'segments (background excluded). This list '
        'corresponds to the class labels in label_file '
        'in ascending order')
    out_file = File(
        'signals.tsv',
        usedefault=True,
        exists=False,
        desc='The name of the file to output to. '
        'signals.tsv by default')
    incl_shared_variance = traits.Bool(
        True,
        usedefault=True,
        desc='By default '
        '(True), returns simple time series calculated from each '
        'region independently (e.g., for noise regression). If '
        'False, returns unique signals for each region, discarding '
        'shared variance (e.g., for connectivity. Only has effect '
        'with 4D probability maps.')
    include_global = traits.Bool(
        False,
        usedefault=True,
        desc='If True, include an extra column '
        'labeled "GlobalSignal", with values calculated from the entire brain '
        '(instead of just regions).')
    detrend = traits.Bool(
        False,
        usedefault=True,
        desc='If True, perform detrending using nilearn.')


class SignalExtractionOutputSpec(TraitedSpec):
    out_file = File(
        exists=True,
        desc='tsv file containing the computed '
        'signals, with as many columns as there are labels and as '
        'many rows as there are timepoints in in_file, plus a '
        'header row with values from class_labels')


class SignalExtraction(NilearnBaseInterface, SimpleInterface):
    '''
    Extracts signals over tissue classes or brain regions

    >>> seinterface = SignalExtraction()
    >>> seinterface.inputs.in_file = 'functional.nii'
    >>> seinterface.inputs.label_files = 'segmentation0.nii.gz'
    >>> seinterface.inputs.out_file = 'means.tsv'
    >>> segments = ['CSF', 'GrayMatter', 'WhiteMatter']
    >>> seinterface.inputs.class_labels = segments
    >>> seinterface.inputs.detrend = True
    >>> seinterface.inputs.include_global = True
    '''
    input_spec = SignalExtractionInputSpec
    output_spec = SignalExtractionOutputSpec

    def _run_interface(self, runtime):
        maskers = self._process_inputs()

        signals = []
        for masker in maskers:
            signals.append(masker.fit_transform(self.inputs.in_file))
        region_signals = np.hstack(signals)

        output = np.vstack((self.inputs.class_labels,
                            region_signals.astype(str)))

        # save output
        self._results['out_file'] = os.path.join(runtime.cwd,
                                                 self.inputs.out_file)
        np.savetxt(
            self._results['out_file'], output, fmt=b'%s', delimiter='\t')
        return runtime

    def _process_inputs(self):
        ''' validate and  process inputs into useful form.
        Returns a list of nilearn maskers and the list of corresponding label
        names.'''
        import nilearn.input_data as nl
        import nilearn.image as nli

        label_data = nli.concat_imgs(self.inputs.label_files)
        maskers = []

        # determine form of label files, choose appropriate nilearn masker
        if np.amax(label_data.get_data()) > 1:  # 3d label file
            n_labels = np.amax(label_data.get_data())
            maskers.append(nl.NiftiLabelsMasker(label_data))
        else:  # 4d labels
            n_labels = label_data.get_data().shape[3]
            if self.inputs.incl_shared_variance:  # independent computation
                for img in nli.iter_img(label_data):
                    maskers.append(
                        nl.NiftiMapsMasker(
                            self._4d(img.get_data(), img.affine)))
            else:  # one computation fitting all
                maskers.append(nl.NiftiMapsMasker(label_data))

        # check label list size
        if not np.isclose(int(n_labels), n_labels):
            raise ValueError(
                'The label files {} contain invalid value {}. Check input.'
                .format(self.inputs.label_files, n_labels))

        if len(self.inputs.class_labels) != n_labels:
            raise ValueError('The length of class_labels {} does not '
                             'match the number of regions {} found in '
                             'label_files {}'.format(self.inputs.class_labels,
                                                     n_labels,
                                                     self.inputs.label_files))

        if self.inputs.include_global:
            global_label_data = label_data.get_data().sum(
                axis=3)  # sum across all regions
            global_label_data = np.rint(global_label_data).astype(int).clip(
                0, 1)  # binarize
            global_label_data = self._4d(global_label_data, label_data.affine)
            global_masker = nl.NiftiLabelsMasker(
                global_label_data, detrend=self.inputs.detrend)
            maskers.insert(0, global_masker)
            self.inputs.class_labels.insert(0, 'GlobalSignal')

        for masker in maskers:
            masker.set_params(detrend=self.inputs.detrend)

        return maskers

    def _4d(self, array, affine):
        ''' takes a 3-dimensional numpy array and an affine,
        returns the equivalent 4th dimensional nifti file '''
        return nb.Nifti1Image(array[:, :, :, np.newaxis], affine)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The spm module provides basic functions for interfacing with matlab
and spm to access spm tools.
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import str, bytes

# Standard library imports
import os
from glob import glob

# Third-party imports
import numpy as np

# Local imports
from ... import logging
from ...utils.filemanip import (ensure_list, simplify_list,
                                split_filename)
from ..base import (Bunch, traits, TraitedSpec, File, Directory,
                    OutputMultiPath, InputMultiPath, isdefined)
from .base import (SPMCommand, SPMCommandInputSpec, scans_for_fnames,
                   ImageFileSPM)

__docformat__ = 'restructuredtext'
iflogger = logging.getLogger('nipype.interface')


class Level1DesignInputSpec(SPMCommandInputSpec):
    spm_mat_dir = Directory(
        exists=True, field='dir', desc='directory to store SPM.mat file (opt)')
    timing_units = traits.Enum(
        'secs',
        'scans',
        field='timing.units',
        desc='units for specification of onsets',
        mandatory=True)
    interscan_interval = traits.Float(
        field='timing.RT', desc='Interscan interval in secs', mandatory=True)
    microtime_resolution = traits.Int(
        field='timing.fmri_t',
        desc=('Number of time-bins per scan '
              'in secs (opt)'))
    microtime_onset = traits.Float(
        field='timing.fmri_t0',
        desc=('The onset/time-bin in seconds for '
              'alignment (opt)'))
    session_info = traits.Any(
        field='sess',
        desc=('Session specific information generated '
              'by ``modelgen.SpecifyModel``'),
        mandatory=True)
    factor_info = traits.List(
        traits.Dict(traits.Enum('name', 'levels')),
        field='fact',
        desc=('Factor specific information '
              'file (opt)'))
    bases = traits.Dict(
        traits.Enum('hrf', 'fourier', 'fourier_han', 'gamma', 'fir'),
        field='bases',
        desc="""
            dict {'name':{'basesparam1':val,...}}
            name : string
                Name of basis function (hrf, fourier, fourier_han,
                gamma, fir)

                hrf :
                    derivs : 2-element list
                        Model  HRF  Derivatives. No derivatives: [0,0],
                        Time derivatives : [1,0], Time and Dispersion
                        derivatives: [1,1]
                fourier, fourier_han, gamma, fir:
                    length : int
                        Post-stimulus window length (in seconds)
                    order : int
                        Number of basis functions
""",
        mandatory=True)
    volterra_expansion_order = traits.Enum(
        1, 2, field='volt', desc=('Model interactions - '
                                  'yes:1, no:2'))
    global_intensity_normalization = traits.Enum(
        'none',
        'scaling',
        field='global',
        desc=('Global intensity '
              'normalization - '
              'scaling or none'))
    mask_image = File(
        exists=True,
        field='mask',
        desc='Image  for  explicitly  masking the analysis')
    mask_threshold = traits.Either(
        traits.Enum('-Inf'),
        traits.Float(),
        desc="Thresholding for the mask",
        default='-Inf',
        usedefault=True)
    model_serial_correlations = traits.Enum(
        'AR(1)',
        'FAST',
        'none',
        field='cvi',
        desc=('Model serial correlations '
              'AR(1), FAST or none. FAST '
              'is available in SPM12'))
    flags = traits.Dict(
        desc='Additional arguments to the job, e.g., a common SPM operation is to '
             'modify the default masking threshold (mthresh)')


class Level1DesignOutputSpec(TraitedSpec):
    spm_mat_file = File(exists=True, desc='SPM mat file')


class Level1Design(SPMCommand):
    """Generate an SPM design matrix

    http://www.fil.ion.ucl.ac.uk/spm/doc/manual.pdf#page=59

    Examples
    --------

    >>> level1design = Level1Design()
    >>> level1design.inputs.timing_units = 'secs'
    >>> level1design.inputs.interscan_interval = 2.5
    >>> level1design.inputs.bases = {'hrf':{'derivs': [0,0]}}
    >>> level1design.inputs.session_info = 'session_info.npz'
    >>> level1design.inputs.flags = {'mthresh': 0.4}
    >>> level1design.run() # doctest: +SKIP

    """

    input_spec = Level1DesignInputSpec
    output_spec = Level1DesignOutputSpec

    _jobtype = 'stats'
    _jobname = 'fmri_spec'

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm
        """
        if opt in ['spm_mat_dir', 'mask_image']:
            return np.array([str(val)], dtype=object)
        if opt in ['session_info']:  # , 'factor_info']:
            if isinstance(val, dict):
                return [val]
            else:
                return val
        return super(Level1Design, self)._format_arg(opt, spec, val)

    def _parse_inputs(self):
        """validate spm realign options if set to None ignore
        """
        einputs = super(Level1Design,
                        self)._parse_inputs(skip=('mask_threshold', 'flags'))
        if isdefined(self.inputs.flags):
            einputs[0].update(
                {flag: val
                 for (flag, val) in self.inputs.flags.items()})
        for sessinfo in einputs[0]['sess']:
            sessinfo['scans'] = scans_for_fnames(
                ensure_list(sessinfo['scans']), keep4d=False)
        if not isdefined(self.inputs.spm_mat_dir):
            einputs[0]['dir'] = np.array([str(os.getcwd())], dtype=object)
        return einputs

    def _make_matlab_command(self, content):
        """validates spm options and generates job structure
        if mfile is True uses matlab .m file
        else generates a job structure and saves in .mat
        """
        if isdefined(self.inputs.mask_image):
            # SPM doesn't handle explicit masking properly, especially
            # when you want to use the entire mask image
            postscript = "load SPM;\n"
            postscript += ("SPM.xM.VM = spm_vol('%s');\n" % simplify_list(
                self.inputs.mask_image))
            postscript += "SPM.xM.I = 0;\n"
            postscript += "SPM.xM.T = [];\n"
            postscript += ("SPM.xM.TH = ones(size(SPM.xM.TH))*(%s);\n" %
                           self.inputs.mask_threshold)
            postscript += ("SPM.xM.xs = struct('Masking', "
                           "'explicit masking only');\n")
            postscript += "save SPM SPM;\n"
        else:
            postscript = None
        return super(Level1Design, self)._make_matlab_command(
            content, postscript=postscript)

    def _list_outputs(self):
        outputs = self._outputs().get()
        spm = os.path.join(os.getcwd(), 'SPM.mat')
        outputs['spm_mat_file'] = spm
        return outputs


class EstimateModelInputSpec(SPMCommandInputSpec):
    spm_mat_file = File(
        exists=True,
        field='spmmat',
        copyfile=True,
        mandatory=True,
        desc='Absolute path to SPM.mat')
    estimation_method = traits.Dict(
        traits.Enum('Classical', 'Bayesian2', 'Bayesian'),
        field='method',
        mandatory=True,
        desc=('Dictionary of either Classical: 1, Bayesian: 1, '
              'or Bayesian2: 1 (dict)'))
    write_residuals = traits.Bool(
        field='write_residuals', desc="Write individual residual images")
    flags = traits.Dict(desc='Additional arguments')


class EstimateModelOutputSpec(TraitedSpec):
    mask_image = ImageFileSPM(
        exists=True, desc='binary mask to constrain estimation')
    beta_images = OutputMultiPath(
        ImageFileSPM(exists=True), desc='design parameter estimates')
    residual_image = ImageFileSPM(
        exists=True, desc='Mean-squared image of the residuals')
    residual_images = OutputMultiPath(
        ImageFileSPM(exists=True),
        desc="individual residual images (requires `write_residuals`")
    RPVimage = ImageFileSPM(exists=True, desc='Resels per voxel image')
    spm_mat_file = File(exists=True, desc='Updated SPM mat file')
    labels = ImageFileSPM(exists=True, desc="label file")
    SDerror = OutputMultiPath(
        ImageFileSPM(exists=True),
        desc="Images of the standard deviation of the error")
    ARcoef = OutputMultiPath(
        ImageFileSPM(exists=True), desc="Images of the AR coefficient")
    Cbetas = OutputMultiPath(
        ImageFileSPM(exists=True), desc="Images of the parameter posteriors")
    SDbetas = OutputMultiPath(
        ImageFileSPM(exists=True),
        desc="Images of the standard deviation of parameter posteriors")


class EstimateModel(SPMCommand):
    """Use spm_spm to estimate the parameters of a model

    http://www.fil.ion.ucl.ac.uk/spm/doc/manual.pdf#page=69

    Examples
    --------
    >>> est = EstimateModel()
    >>> est.inputs.spm_mat_file = 'SPM.mat'
    >>> est.inputs.estimation_method = {'Classical': 1}
    >>> est.run() # doctest: +SKIP
    """
    input_spec = EstimateModelInputSpec
    output_spec = EstimateModelOutputSpec
    _jobtype = 'stats'
    _jobname = 'fmri_est'

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm
        """
        if opt == 'spm_mat_file':
            return np.array([str(val)], dtype=object)
        if opt == 'estimation_method':
            if isinstance(val, (str, bytes)):
                return {'{}'.format(val): 1}
            else:
                return val
        return super(EstimateModel, self)._format_arg(opt, spec, val)

    def _parse_inputs(self):
        """validate spm realign options if set to None ignore
        """
        einputs = super(EstimateModel, self)._parse_inputs(skip=('flags'))
        if isdefined(self.inputs.flags):
            einputs[0].update(
                {flag: val
                 for (flag, val) in self.inputs.flags.items()})
        return einputs

    def _list_outputs(self):
        import scipy.io as sio
        outputs = self._outputs().get()
        pth = os.path.dirname(self.inputs.spm_mat_file)
        outtype = 'nii' if '12' in self.version.split('.')[0] else 'img'
        spm = sio.loadmat(self.inputs.spm_mat_file, struct_as_record=False)

        betas = [vbeta.fname[0] for vbeta in spm['SPM'][0, 0].Vbeta[0]]
        if ('Bayesian' in self.inputs.estimation_method.keys()
                or 'Bayesian2' in self.inputs.estimation_method.keys()):
            outputs['labels'] = os.path.join(pth, 'labels.{}'.format(outtype))
            outputs['SDerror'] = glob(os.path.join(pth, 'Sess*_SDerror*'))
            outputs['ARcoef'] = glob(os.path.join(pth, 'Sess*_AR_*'))
            if betas:
                outputs['Cbetas'] = [
                    os.path.join(pth, 'C{}'.format(beta)) for beta in betas
                ]
                outputs['SDbetas'] = [
                    os.path.join(pth, 'SD{}'.format(beta)) for beta in betas
                ]

        if 'Classical' in self.inputs.estimation_method.keys():
            outputs['residual_image'] = os.path.join(
                pth, 'ResMS.{}'.format(outtype))
            outputs['RPVimage'] = os.path.join(pth, 'RPV.{}'.format(outtype))
            if self.inputs.write_residuals:
                outputs['residual_images'] = glob(os.path.join(pth, 'Res_*'))
            if betas:
                outputs['beta_images'] = [
                    os.path.join(pth, beta) for beta in betas
                ]

        outputs['mask_image'] = os.path.join(pth, 'mask.{}'.format(outtype))
        outputs['spm_mat_file'] = os.path.join(pth, 'SPM.mat')
        return outputs


class EstimateContrastInputSpec(SPMCommandInputSpec):
    spm_mat_file = File(
        exists=True,
        field='spmmat',
        desc='Absolute path to SPM.mat',
        copyfile=True,
        mandatory=True)
    contrasts = traits.List(
        traits.Either(
            traits.Tuple(traits.Str, traits.Enum('T'), traits.List(traits.Str),
                         traits.List(traits.Float)),
            traits.Tuple(traits.Str, traits.Enum('T'), traits.List(traits.Str),
                         traits.List(traits.Float), traits.List(traits.Float)),
            traits.Tuple(traits.Str, traits.Enum('F'),
                         traits.List(
                             traits.Either(
                                 traits.Tuple(traits.Str, traits.Enum('T'),
                                              traits.List(traits.Str),
                                              traits.List(traits.Float)),
                                 traits.Tuple(traits.Str, traits.Enum('T'),
                                              traits.List(traits.Str),
                                              traits.List(traits.Float),
                                              traits.List(traits.Float)))))),
        desc="""List of contrasts with each contrast being a list of the form:
            [('name', 'stat', [condition list], [weight list], [session list])]
            If session list is None or not provided, all sessions are used. For
            F contrasts, the condition list should contain previously defined
            T-contrasts.""",
        mandatory=True)
    beta_images = InputMultiPath(
        File(exists=True),
        desc=('Parameter estimates of the '
              'design matrix'),
        copyfile=False,
        mandatory=True)
    residual_image = File(
        exists=True,
        desc='Mean-squared image of the residuals',
        copyfile=False,
        mandatory=True)
    use_derivs = traits.Bool(
        desc='use derivatives for estimation', xor=['group_contrast'])
    group_contrast = traits.Bool(
        desc='higher level contrast', xor=['use_derivs'])


class EstimateContrastOutputSpec(TraitedSpec):
    con_images = OutputMultiPath(
        File(exists=True), desc='contrast images from a t-contrast')
    spmT_images = OutputMultiPath(
        File(exists=True), desc='stat images from a t-contrast')
    ess_images = OutputMultiPath(
        File(exists=True), desc='contrast images from an F-contrast')
    spmF_images = OutputMultiPath(
        File(exists=True), desc='stat images from an F-contrast')
    spm_mat_file = File(exists=True, desc='Updated SPM mat file')


class EstimateContrast(SPMCommand):
    """Use spm_contrasts to estimate contrasts of interest

    Examples
    --------
    >>> import nipype.interfaces.spm as spm
    >>> est = spm.EstimateContrast()
    >>> est.inputs.spm_mat_file = 'SPM.mat'
    >>> cont1 = ('Task>Baseline','T', ['Task-Odd','Task-Even'],[0.5,0.5])
    >>> cont2 = ('Task-Odd>Task-Even','T', ['Task-Odd','Task-Even'],[1,-1])
    >>> contrasts = [cont1,cont2]
    >>> est.inputs.contrasts = contrasts
    >>> est.run() # doctest: +SKIP

    """

    input_spec = EstimateContrastInputSpec
    output_spec = EstimateContrastOutputSpec
    _jobtype = 'stats'
    _jobname = 'con'

    def _make_matlab_command(self, _):
        """validates spm options and generates job structure
        """
        contrasts = []
        cname = []
        for i, cont in enumerate(self.inputs.contrasts):
            cname.insert(i, cont[0])
            contrasts.insert(i,
                             Bunch(
                                 name=cont[0],
                                 stat=cont[1],
                                 conditions=cont[2],
                                 weights=None,
                                 sessions=None))
            if len(cont) >= 4:
                contrasts[i].weights = cont[3]
            if len(cont) >= 5:
                contrasts[i].sessions = cont[4]
        script = "% generated by nipype.interfaces.spm\n"
        script += "spm_defaults;\n"
        script += ("jobs{1}.stats{1}.con.spmmat  = {'%s'};\n" %
                   self.inputs.spm_mat_file)
        script += "load(jobs{1}.stats{1}.con.spmmat{:});\n"
        script += "SPM.swd = '%s';\n" % os.getcwd()
        script += "save(jobs{1}.stats{1}.con.spmmat{:},'SPM');\n"
        script += "names = SPM.xX.name;\n"
        # get names for columns
        if (isdefined(self.inputs.group_contrast)
                and self.inputs.group_contrast):
            script += "condnames=names;\n"
        else:
            if self.inputs.use_derivs:
                script += "pat = 'Sn\([0-9]*\) (.*)';\n"
            else:
                script += ("pat = 'Sn\([0-9]*\) (.*)\*bf\(1\)|Sn\([0-9]*\) "
                           ".*\*bf\([2-9]\)|Sn\([0-9]*\) (.*)';\n")
            script += "t = regexp(names,pat,'tokens');\n"
            # get sessidx for columns
            script += "pat1 = 'Sn\(([0-9].*)\)\s.*';\n"
            script += "t1 = regexp(names,pat1,'tokens');\n"
            script += ("for i0=1:numel(t),condnames{i0}='';condsess(i0)=0;if "
                       "~isempty(t{i0}{1}),condnames{i0} = t{i0}{1}{1};"
                       "condsess(i0)=str2num(t1{i0}{1}{1});end;end;\n")
        # BUILD CONTRAST SESSION STRUCTURE
        for i, contrast in enumerate(contrasts):
            if contrast.stat == 'T':
                script += ("consess{%d}.tcon.name   = '%s';\n" %
                           (i + 1, contrast.name))
                script += (
                    "consess{%d}.tcon.convec = zeros(1,numel(names));\n" %
                    (i + 1))
                for c0, cond in enumerate(contrast.conditions):
                    script += ("idx = strmatch('%s',condnames,'exact');\n" %
                               (cond))
                    script += (("if isempty(idx), throw(MException("
                                "'CondName:Chk', sprintf('Condition %%s not "
                                "found in design','%s'))); end;\n") % cond)
                    if contrast.sessions:
                        for sno, sw in enumerate(contrast.sessions):
                            script += ("sidx = find(condsess(idx)==%d);\n" %
                                       (sno + 1))
                            script += (("consess{%d}.tcon.convec(idx(sidx)) "
                                        "= %f;\n") %
                                       (i + 1, sw * contrast.weights[c0]))
                    else:
                        script += ("consess{%d}.tcon.convec(idx) = %f;\n" %
                                   (i + 1, contrast.weights[c0]))
        for i, contrast in enumerate(contrasts):
            if contrast.stat == 'F':
                script += ("consess{%d}.fcon.name   =  '%s';\n" %
                           (i + 1, contrast.name))
                for cl0, fcont in enumerate(contrast.conditions):
                    try:
                        tidx = cname.index(fcont[0])
                    except:
                        Exception("Contrast Estimate: could not get index of"
                                  " T contrast. probably not defined prior "
                                  "to the F contrasts")
                    script += (("consess{%d}.fcon.convec{%d} = "
                                "consess{%d}.tcon.convec;\n") %
                               (i + 1, cl0 + 1, tidx + 1))
        script += "jobs{1}.stats{1}.con.consess = consess;\n"
        script += ("if strcmp(spm('ver'),'SPM8'), spm_jobman('initcfg');"
                   "jobs=spm_jobman('spm5tospm8',{jobs});end\n")
        script += "spm_jobman('run',jobs);"
        return script

    def _list_outputs(self):
        import scipy.io as sio
        outputs = self._outputs().get()
        pth, _ = os.path.split(self.inputs.spm_mat_file)
        spm = sio.loadmat(self.inputs.spm_mat_file, struct_as_record=False)
        con_images = []
        spmT_images = []
        for con in spm['SPM'][0, 0].xCon[0]:
            con_images.append(str(os.path.join(pth, con.Vcon[0, 0].fname[0])))
            spmT_images.append(str(os.path.join(pth, con.Vspm[0, 0].fname[0])))
        if con_images:
            outputs['con_images'] = con_images
            outputs['spmT_images'] = spmT_images
        spm12 = '12' in self.version.split('.')[0]
        if spm12:
            ess = glob(os.path.join(pth, 'ess*.nii'))
        else:
            ess = glob(os.path.join(pth, 'ess*.img'))
        if len(ess) > 0:
            outputs['ess_images'] = sorted(ess)
        if spm12:
            spmf = glob(os.path.join(pth, 'spmF*.nii'))
        else:
            spmf = glob(os.path.join(pth, 'spmF*.img'))
        if len(spmf) > 0:
            outputs['spmF_images'] = sorted(spmf)
        outputs['spm_mat_file'] = self.inputs.spm_mat_file
        return outputs


class ThresholdInputSpec(SPMCommandInputSpec):
    spm_mat_file = File(
        exists=True,
        desc='absolute path to SPM.mat',
        copyfile=True,
        mandatory=True)
    stat_image = File(
        exists=True, desc='stat image', copyfile=False, mandatory=True)
    contrast_index = traits.Int(
        mandatory=True, desc='which contrast in the SPM.mat to use')
    use_fwe_correction = traits.Bool(
        True,
        usedefault=True,
        desc=('whether to use FWE (Bonferroni) '
              'correction for initial threshold '
              '(height_threshold_type has to be '
              'set to p-value)'))
    use_topo_fdr = traits.Bool(
        True,
        usedefault=True,
        desc=('whether to use FDR over cluster extent '
              'probabilities'))
    height_threshold = traits.Float(
        0.05,
        usedefault=True,
        desc=('value for initial thresholding '
              '(defining clusters)'))
    height_threshold_type = traits.Enum(
        'p-value',
        'stat',
        usedefault=True,
        desc=('Is the cluster forming '
              'threshold a stat value or '
              'p-value?'))
    extent_fdr_p_threshold = traits.Float(
        0.05,
        usedefault=True,
        desc=('p threshold on FDR corrected '
              'cluster size probabilities'))
    extent_threshold = traits.Int(
        0, usedefault=True, desc='Minimum cluster size in voxels')
    force_activation = traits.Bool(
        False,
        usedefault=True,
        desc=('In case no clusters survive the '
              'topological inference step this '
              'will pick a culster with the highes '
              'sum of t-values. Use with care.'))


class ThresholdOutputSpec(TraitedSpec):
    thresholded_map = File(exists=True)
    n_clusters = traits.Int()
    pre_topo_fdr_map = File(exists=True)
    pre_topo_n_clusters = traits.Int()
    activation_forced = traits.Bool()
    cluster_forming_thr = traits.Float()


class Threshold(SPMCommand):
    """Topological FDR thresholding based on cluster extent/size. Smoothness is
    estimated from GLM residuals but is assumed to be the same for all of the
    voxels.

    Examples
    --------

    >>> thresh = Threshold()
    >>> thresh.inputs.spm_mat_file = 'SPM.mat'
    >>> thresh.inputs.stat_image = 'spmT_0001.img'
    >>> thresh.inputs.contrast_index = 1
    >>> thresh.inputs.extent_fdr_p_threshold = 0.05
    >>> thresh.run() # doctest: +SKIP
    """
    input_spec = ThresholdInputSpec
    output_spec = ThresholdOutputSpec

    def _gen_thresholded_map_filename(self):
        _, fname, ext = split_filename(self.inputs.stat_image)
        return os.path.abspath(fname + "_thr" + ext)

    def _gen_pre_topo_map_filename(self):
        _, fname, ext = split_filename(self.inputs.stat_image)
        return os.path.abspath(fname + "_pre_topo_thr" + ext)

    def _make_matlab_command(self, _):
        script = "con_index = %d;\n" % self.inputs.contrast_index
        script += "cluster_forming_thr = %f;\n" % self.inputs.height_threshold
        if self.inputs.use_fwe_correction:
            script += "thresDesc  = 'FWE';\n"
        else:
            script += "thresDesc  = 'none';\n"

        if self.inputs.use_topo_fdr:
            script += "use_topo_fdr  = 1;\n"
        else:
            script += "use_topo_fdr  = 0;\n"

        if self.inputs.force_activation:
            script += "force_activation  = 1;\n"
        else:
            script += "force_activation  = 0;\n"
        script += ("cluster_extent_p_fdr_thr = %f;\n" %
                   self.inputs.extent_fdr_p_threshold)
        script += "stat_filename = '%s';\n" % self.inputs.stat_image
        script += ("height_threshold_type = '%s';\n" %
                   self.inputs.height_threshold_type)
        script += "extent_threshold = %d;\n" % self.inputs.extent_threshold

        script += "load %s;\n" % self.inputs.spm_mat_file
        script += """
FWHM  = SPM.xVol.FWHM;
df = [SPM.xCon(con_index).eidf SPM.xX.erdf];
STAT = SPM.xCon(con_index).STAT;
R = SPM.xVol.R;
S = SPM.xVol.S;
n = 1;

switch thresDesc
    case 'FWE'
        cluster_forming_thr = spm_uc(cluster_forming_thr,df,STAT,R,n,S);

    case 'none'
        if strcmp(height_threshold_type, 'p-value')
            cluster_forming_thr = spm_u(cluster_forming_thr^(1/n),df,STAT);
        end
end

stat_map_vol = spm_vol(stat_filename);
[stat_map_data, stat_map_XYZmm] = spm_read_vols(stat_map_vol);

Z = stat_map_data(:)';
[x,y,z] = ind2sub(size(stat_map_data),(1:numel(stat_map_data))');
XYZ = cat(1, x', y', z');

XYZth = XYZ(:, Z >= cluster_forming_thr);
Zth = Z(Z >= cluster_forming_thr);

"""
        script += (("spm_write_filtered(Zth,XYZth,stat_map_vol.dim',"
                    "stat_map_vol.mat,'thresholded map', '%s');\n") %
                   self._gen_pre_topo_map_filename())
        script += """
max_size = 0;
max_size_index = 0;
th_nclusters = 0;
nclusters = 0;
if isempty(XYZth)
    thresholded_XYZ = [];
    thresholded_Z = [];
else
    if use_topo_fdr
        V2R        = 1/prod(FWHM(stat_map_vol.dim > 1));
        [uc,Pc,ue] = spm_uc_clusterFDR(cluster_extent_p_fdr_thr,df,STAT,R,n,Z,XYZ,V2R,cluster_forming_thr);
    end

    voxel_labels = spm_clusters(XYZth);
    nclusters = max(voxel_labels);

    thresholded_XYZ = [];
    thresholded_Z = [];

    for i = 1:nclusters
        cluster_size = sum(voxel_labels==i);
         if cluster_size > extent_threshold && (~use_topo_fdr || (cluster_size - uc) > -1)
            thresholded_XYZ = cat(2, thresholded_XYZ, XYZth(:,voxel_labels == i));
            thresholded_Z = cat(2, thresholded_Z, Zth(voxel_labels == i));
            th_nclusters = th_nclusters + 1;
         end
        if force_activation
            cluster_sum = sum(Zth(voxel_labels == i));
            if cluster_sum > max_size
                max_size = cluster_sum;
                max_size_index = i;
            end
        end
    end
end

activation_forced = 0;
if isempty(thresholded_XYZ)
    if force_activation && max_size ~= 0
        thresholded_XYZ = XYZth(:,voxel_labels == max_size_index);
        thresholded_Z = Zth(voxel_labels == max_size_index);
        th_nclusters = 1;
        activation_forced = 1;
    else
        thresholded_Z = [0];
        thresholded_XYZ = [1 1 1]';
        th_nclusters = 0;
    end
end

fprintf('activation_forced = %d\\n',activation_forced);
fprintf('pre_topo_n_clusters = %d\\n',nclusters);
fprintf('n_clusters = %d\\n',th_nclusters);
fprintf('cluster_forming_thr = %f\\n',cluster_forming_thr);

"""
        script += (("spm_write_filtered(thresholded_Z,thresholded_XYZ,"
                    "stat_map_vol.dim',stat_map_vol.mat,'thresholded map',"
                    " '%s');\n") % self._gen_thresholded_map_filename())

        return script

    def aggregate_outputs(self, runtime=None):
        outputs = self._outputs()
        setattr(outputs, 'thresholded_map',
                self._gen_thresholded_map_filename())
        setattr(outputs, 'pre_topo_fdr_map', self._gen_pre_topo_map_filename())
        for line in runtime.stdout.split('\n'):
            if line.startswith("activation_forced = "):
                setattr(outputs, 'activation_forced',
                        line[len("activation_forced = "):].strip() == "1")
            elif line.startswith("n_clusters = "):
                setattr(outputs, 'n_clusters',
                        int(line[len("n_clusters = "):].strip()))
            elif line.startswith("pre_topo_n_clusters = "):
                setattr(outputs, 'pre_topo_n_clusters',
                        int(line[len("pre_topo_n_clusters = "):].strip()))
            elif line.startswith("cluster_forming_thr = "):
                setattr(outputs, 'cluster_forming_thr',
                        float(line[len("cluster_forming_thr = "):].strip()))
        return outputs

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['thresholded_map'] = self._gen_thresholded_map_filename()
        outputs['pre_topo_fdr_map'] = self._gen_pre_topo_map_filename()
        return outputs


class ThresholdStatisticsInputSpec(SPMCommandInputSpec):
    spm_mat_file = File(
        exists=True,
        desc='absolute path to SPM.mat',
        copyfile=True,
        mandatory=True)
    stat_image = File(
        exists=True, desc='stat image', copyfile=False, mandatory=True)
    contrast_index = traits.Int(
        mandatory=True, desc='which contrast in the SPM.mat to use')
    height_threshold = traits.Float(
        desc=('stat value for initial '
              'thresholding (defining clusters)'),
        mandatory=True)
    extent_threshold = traits.Int(
        0, usedefault=True, desc="Minimum cluster size in voxels")


class ThresholdStatisticsOutputSpec(TraitedSpec):
    voxelwise_P_Bonf = traits.Float()
    voxelwise_P_RF = traits.Float()
    voxelwise_P_uncor = traits.Float()
    voxelwise_P_FDR = traits.Float()
    clusterwise_P_RF = traits.Float()
    clusterwise_P_FDR = traits.Float()


class ThresholdStatistics(SPMCommand):
    """Given height and cluster size threshold calculate theoretical
    probabilities concerning false positives

    Examples
    --------

    >>> thresh = ThresholdStatistics()
    >>> thresh.inputs.spm_mat_file = 'SPM.mat'
    >>> thresh.inputs.stat_image = 'spmT_0001.img'
    >>> thresh.inputs.contrast_index = 1
    >>> thresh.inputs.height_threshold = 4.56
    >>> thresh.run() # doctest: +SKIP
    """
    input_spec = ThresholdStatisticsInputSpec
    output_spec = ThresholdStatisticsOutputSpec

    def _make_matlab_command(self, _):
        script = "con_index = %d;\n" % self.inputs.contrast_index
        script += "cluster_forming_thr = %f;\n" % self.inputs.height_threshold
        script += "stat_filename = '%s';\n" % self.inputs.stat_image
        script += "extent_threshold = %d;\n" % self.inputs.extent_threshold
        script += "load '%s'\n" % self.inputs.spm_mat_file
        script += """
FWHM  = SPM.xVol.FWHM;
df = [SPM.xCon(con_index).eidf SPM.xX.erdf];
STAT = SPM.xCon(con_index).STAT;
R = SPM.xVol.R;
S = SPM.xVol.S;
n = 1;

voxelwise_P_Bonf = spm_P_Bonf(cluster_forming_thr,df,STAT,S,n)
voxelwise_P_RF = spm_P_RF(1,0,cluster_forming_thr,df,STAT,R,n)

stat_map_vol = spm_vol(stat_filename);
[stat_map_data, stat_map_XYZmm] = spm_read_vols(stat_map_vol);

Z = stat_map_data(:);
Zum = Z;

        switch STAT
            case 'Z'
                VPs = (1-spm_Ncdf(Zum)).^n;
                voxelwise_P_uncor = (1-spm_Ncdf(cluster_forming_thr)).^n
            case 'T'
                VPs = (1 - spm_Tcdf(Zum,df(2))).^n;
                voxelwise_P_uncor = (1 - spm_Tcdf(cluster_forming_thr,df(2))).^n
            case 'X'
                VPs = (1-spm_Xcdf(Zum,df(2))).^n;
                voxelwise_P_uncor = (1-spm_Xcdf(cluster_forming_thr,df(2))).^n
            case 'F'
                VPs = (1 - spm_Fcdf(Zum,df)).^n;
                voxelwise_P_uncor = (1 - spm_Fcdf(cluster_forming_thr,df)).^n
        end
        VPs = sort(VPs);

voxelwise_P_FDR = spm_P_FDR(cluster_forming_thr,df,STAT,n,VPs)

V2R        = 1/prod(FWHM(stat_map_vol.dim > 1));

clusterwise_P_RF = spm_P_RF(1,extent_threshold*V2R,cluster_forming_thr,df,STAT,R,n)

[x,y,z] = ind2sub(size(stat_map_data),(1:numel(stat_map_data))');
XYZ = cat(1, x', y', z');

[u, CPs, ue] = spm_uc_clusterFDR(0.05,df,STAT,R,n,Z,XYZ,V2R,cluster_forming_thr);

clusterwise_P_FDR = spm_P_clusterFDR(extent_threshold*V2R,df,STAT,R,n,cluster_forming_thr,CPs')
"""
        return script

    def aggregate_outputs(self, runtime=None, needed_outputs=None):
        outputs = self._outputs()
        cur_output = ""
        for line in runtime.stdout.split('\n'):
            if cur_output != "" and len(line.split()) != 0:
                setattr(outputs, cur_output, float(line))
                cur_output = ""
                continue
            if (len(line.split()) != 0 and line.split()[0] in [
                    "clusterwise_P_FDR", "clusterwise_P_RF",
                    "voxelwise_P_Bonf", "voxelwise_P_FDR", "voxelwise_P_RF",
                    "voxelwise_P_uncor"
            ]):
                cur_output = line.split()[0]
                continue

        return outputs


class FactorialDesignInputSpec(SPMCommandInputSpec):
    spm_mat_dir = Directory(
        exists=True, field='dir', desc='directory to store SPM.mat file (opt)')
    # Need to make an alias of InputMultiPath; the inputs below are not Path
    covariates = InputMultiPath(
        traits.Dict(
            key_trait=traits.Enum('vector', 'name', 'interaction',
                                  'centering')),
        field='cov',
        desc=('covariate dictionary {vector, name, '
              'interaction, centering}'))
    threshold_mask_none = traits.Bool(
        field='masking.tm.tm_none',
        xor=['threshold_mask_absolute', 'threshold_mask_relative'],
        desc='do not use threshold masking')
    threshold_mask_absolute = traits.Float(
        field='masking.tm.tma.athresh',
        xor=['threshold_mask_none', 'threshold_mask_relative'],
        desc='use an absolute threshold')
    threshold_mask_relative = traits.Float(
        field='masking.tm.tmr.rthresh',
        xor=['threshold_mask_absolute', 'threshold_mask_none'],
        desc=('threshold using a '
              'proportion of the global '
              'value'))
    use_implicit_threshold = traits.Bool(
        field='masking.im',
        desc=('use implicit mask NaNs or '
              'zeros to threshold'))
    explicit_mask_file = File(
        field='masking.em',  # requires cell
        desc='use an implicit mask file to threshold')
    global_calc_omit = traits.Bool(
        field='globalc.g_omit',
        xor=['global_calc_mean', 'global_calc_values'],
        desc='omit global calculation')
    global_calc_mean = traits.Bool(
        field='globalc.g_mean',
        xor=['global_calc_omit', 'global_calc_values'],
        desc='use mean for global calculation')
    global_calc_values = traits.List(
        traits.Float,
        field='globalc.g_user.global_uval',
        xor=['global_calc_mean', 'global_calc_omit'],
        desc='omit global calculation')
    no_grand_mean_scaling = traits.Bool(
        field='globalm.gmsca.gmsca_no',
        desc=('do not perform grand mean '
              'scaling'))
    global_normalization = traits.Enum(
        1,
        2,
        3,
        field='globalm.glonorm',
        desc=('global normalization None-1, '
              'Proportional-2, ANCOVA-3'))


class FactorialDesignOutputSpec(TraitedSpec):
    spm_mat_file = File(exists=True, desc='SPM mat file')


class FactorialDesign(SPMCommand):
    """Base class for factorial designs

    http://www.fil.ion.ucl.ac.uk/spm/doc/manual.pdf#page=77

    """

    input_spec = FactorialDesignInputSpec
    output_spec = FactorialDesignOutputSpec
    _jobtype = 'stats'
    _jobname = 'factorial_design'

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm
        """
        if opt in ['spm_mat_dir', 'explicit_mask_file']:
            return np.array([str(val)], dtype=object)
        if opt in ['covariates']:
            outlist = []
            mapping = {
                'name': 'cname',
                'vector': 'c',
                'interaction': 'iCFI',
                'centering': 'iCC'
            }
            for dictitem in val:
                outdict = {}
                for key, keyval in list(dictitem.items()):
                    outdict[mapping[key]] = keyval
                outlist.append(outdict)
            return outlist
        return super(FactorialDesign, self)._format_arg(opt, spec, val)

    def _parse_inputs(self):
        """validate spm realign options if set to None ignore
        """
        einputs = super(FactorialDesign, self)._parse_inputs()
        if not isdefined(self.inputs.spm_mat_dir):
            einputs[0]['dir'] = np.array([str(os.getcwd())], dtype=object)
        return einputs

    def _list_outputs(self):
        outputs = self._outputs().get()
        spm = os.path.join(os.getcwd(), 'SPM.mat')
        outputs['spm_mat_file'] = spm
        return outputs


class OneSampleTTestDesignInputSpec(FactorialDesignInputSpec):
    in_files = traits.List(
        File(exists=True),
        field='des.t1.scans',
        mandatory=True,
        minlen=2,
        desc='input files')


class OneSampleTTestDesign(FactorialDesign):
    """Create SPM design for one sample t-test

    Examples
    --------

    >>> ttest = OneSampleTTestDesign()
    >>> ttest.inputs.in_files = ['cont1.nii', 'cont2.nii']
    >>> ttest.run() # doctest: +SKIP
    """

    input_spec = OneSampleTTestDesignInputSpec

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm
        """
        if opt in ['in_files']:
            return np.array(val, dtype=object)
        return super(OneSampleTTestDesign, self)._format_arg(opt, spec, val)


class TwoSampleTTestDesignInputSpec(FactorialDesignInputSpec):
    # very unlikely that you will have a single image in one group, so setting
    # parameters to require at least two files in each group [SG]
    group1_files = traits.List(
        File(exists=True),
        field='des.t2.scans1',
        mandatory=True,
        minlen=2,
        desc='Group 1 input files')
    group2_files = traits.List(
        File(exists=True),
        field='des.t2.scans2',
        mandatory=True,
        minlen=2,
        desc='Group 2 input files')
    dependent = traits.Bool(
        field='des.t2.dept',
        desc=('Are the measurements dependent between '
              'levels'))
    unequal_variance = traits.Bool(
        field='des.t2.variance',
        desc=('Are the variances equal or unequal '
              'between groups'))


class TwoSampleTTestDesign(FactorialDesign):
    """Create SPM design for two sample t-test

    Examples
    --------

    >>> ttest = TwoSampleTTestDesign()
    >>> ttest.inputs.group1_files = ['cont1.nii', 'cont2.nii']
    >>> ttest.inputs.group2_files = ['cont1a.nii', 'cont2a.nii']
    >>> ttest.run() # doctest: +SKIP
    """

    input_spec = TwoSampleTTestDesignInputSpec

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm
        """
        if opt in ['group1_files', 'group2_files']:
            return np.array(val, dtype=object)
        return super(TwoSampleTTestDesign, self)._format_arg(opt, spec, val)


class PairedTTestDesignInputSpec(FactorialDesignInputSpec):
    paired_files = traits.List(
        traits.List(File(exists=True), minlen=2, maxlen=2),
        field='des.pt.pair',
        mandatory=True,
        minlen=2,
        desc='List of paired files')
    grand_mean_scaling = traits.Bool(
        field='des.pt.gmsca', desc='Perform grand mean scaling')
    ancova = traits.Bool(
        field='des.pt.ancova', desc='Specify ancova-by-factor regressors')


class PairedTTestDesign(FactorialDesign):
    """Create SPM design for paired t-test

    Examples
    --------

    >>> pttest = PairedTTestDesign()
    >>> pttest.inputs.paired_files = [['cont1.nii','cont1a.nii'],['cont2.nii','cont2a.nii']]
    >>> pttest.run() # doctest: +SKIP
    """

    input_spec = PairedTTestDesignInputSpec

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm
        """
        if opt in ['paired_files']:
            return [dict(scans=np.array(files, dtype=object)) for files in val]
        return super(PairedTTestDesign, self)._format_arg(opt, spec, val)


class MultipleRegressionDesignInputSpec(FactorialDesignInputSpec):
    in_files = traits.List(
        File(exists=True),
        field='des.mreg.scans',
        mandatory=True,
        minlen=2,
        desc='List of files')
    include_intercept = traits.Bool(
        True,
        field='des.mreg.incint',
        usedefault=True,
        desc='Include intercept in design')
    user_covariates = InputMultiPath(
        traits.Dict(key_trait=traits.Enum('vector', 'name', 'centering')),
        field='des.mreg.mcov',
        desc=('covariate dictionary {vector, '
              'name, centering}'))


class MultipleRegressionDesign(FactorialDesign):
    """Create SPM design for multiple regression

    Examples
    --------

    >>> mreg = MultipleRegressionDesign()
    >>> mreg.inputs.in_files = ['cont1.nii','cont2.nii']
    >>> mreg.run() # doctest: +SKIP
    """

    input_spec = MultipleRegressionDesignInputSpec

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm
        """
        if opt in ['in_files']:
            return np.array(val, dtype=object)
        if opt in ['user_covariates']:
            outlist = []
            mapping = {'name': 'cname', 'vector': 'c', 'centering': 'iCC'}
            for dictitem in val:
                outdict = {}
                for key, keyval in list(dictitem.items()):
                    outdict[mapping[key]] = keyval
                outlist.append(outdict)
            return outlist
        return (super(MultipleRegressionDesign, self)._format_arg(
            opt, spec, val))



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import ExpertAutomatedRegistration


def test_ExpertAutomatedRegistration_inputs():
    input_map = dict(
        affineMaxIterations=dict(argstr='--affineMaxIterations %d', ),
        affineSamplingRatio=dict(argstr='--affineSamplingRatio %f', ),
        args=dict(argstr='%s', ),
        bsplineMaxIterations=dict(argstr='--bsplineMaxIterations %d', ),
        bsplineSamplingRatio=dict(argstr='--bsplineSamplingRatio %f', ),
        controlPointSpacing=dict(argstr='--controlPointSpacing %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expectedOffset=dict(argstr='--expectedOffset %f', ),
        expectedRotation=dict(argstr='--expectedRotation %f', ),
        expectedScale=dict(argstr='--expectedScale %f', ),
        expectedSkew=dict(argstr='--expectedSkew %f', ),
        fixedImage=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        fixedImageMask=dict(
            argstr='--fixedImageMask %s',
            extensions=None,
        ),
        fixedLandmarks=dict(argstr='--fixedLandmarks %s...', ),
        initialization=dict(argstr='--initialization %s', ),
        interpolation=dict(argstr='--interpolation %s', ),
        loadTransform=dict(
            argstr='--loadTransform %s',
            extensions=None,
        ),
        metric=dict(argstr='--metric %s', ),
        minimizeMemory=dict(argstr='--minimizeMemory ', ),
        movingImage=dict(
            argstr='%s',
            extensions=None,
            position=-1,
        ),
        movingLandmarks=dict(argstr='--movingLandmarks %s...', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        randomNumberSeed=dict(argstr='--randomNumberSeed %d', ),
        registration=dict(argstr='--registration %s', ),
        resampledImage=dict(
            argstr='--resampledImage %s',
            hash_files=False,
        ),
        rigidMaxIterations=dict(argstr='--rigidMaxIterations %d', ),
        rigidSamplingRatio=dict(argstr='--rigidSamplingRatio %f', ),
        sampleFromOverlap=dict(argstr='--sampleFromOverlap ', ),
        saveTransform=dict(
            argstr='--saveTransform %s',
            hash_files=False,
        ),
        verbosityLevel=dict(argstr='--verbosityLevel %s', ),
    )
    inputs = ExpertAutomatedRegistration.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ExpertAutomatedRegistration_outputs():
    output_map = dict(
        resampledImage=dict(extensions=None, ),
        saveTransform=dict(extensions=None, ),
    )
    outputs = ExpertAutomatedRegistration.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..asl import FitAsl


def test_FitAsl_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        cbf_file=dict(
            argstr='-cbf %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_cbf.nii.gz',
        ),
        dpld=dict(argstr='-dPLD %f', ),
        dt_inv2=dict(argstr='-dTinv2 %f', ),
        eff=dict(argstr='-eff %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        error_file=dict(
            argstr='-error %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_error.nii.gz',
        ),
        gm_plasma=dict(argstr='-gmL %f', ),
        gm_t1=dict(argstr='-gmT1 %f', ),
        gm_ttt=dict(argstr='-gmTTT %f', ),
        ir_output=dict(
            argstr='-IRoutput %s',
            extensions=None,
        ),
        ir_volume=dict(
            argstr='-IRvolume %s',
            extensions=None,
        ),
        ldd=dict(argstr='-LDD %f', ),
        m0map=dict(
            argstr='-m0map %s',
            extensions=None,
        ),
        m0mape=dict(
            argstr='-m0mape %s',
            extensions=None,
        ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
            position=2,
        ),
        mul=dict(argstr='-mul %f', ),
        mulgm=dict(argstr='-sig', ),
        out=dict(argstr='-out %f', ),
        pasl=dict(argstr='-pasl', ),
        pcasl=dict(argstr='-pcasl', ),
        plasma_coeff=dict(argstr='-L %f', ),
        pld=dict(argstr='-PLD %f', ),
        pv0=dict(argstr='-pv0 %d', ),
        pv2=dict(argstr='-pv2 %d', ),
        pv3=dict(argstr='-pv3 %d %d %d', ),
        pv_threshold=dict(argstr='-pvthreshold', ),
        seg=dict(
            argstr='-seg %s',
            extensions=None,
        ),
        segstyle=dict(argstr='-segstyle', ),
        sig=dict(argstr='-sig', ),
        source_file=dict(
            argstr='-source %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        syn_file=dict(
            argstr='-syn %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_syn.nii.gz',
        ),
        t1_art_cmp=dict(argstr='-T1a %f', ),
        t1map=dict(
            argstr='-t1map %s',
            extensions=None,
        ),
        t_inv1=dict(argstr='-Tinv1 %f', ),
        t_inv2=dict(argstr='-Tinv2 %f', ),
        wm_plasma=dict(argstr='-wmL %f', ),
        wm_t1=dict(argstr='-wmT1 %f', ),
        wm_ttt=dict(argstr='-wmTTT %f', ),
    )
    inputs = FitAsl.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FitAsl_outputs():
    output_map = dict(
        cbf_file=dict(extensions=None, ),
        error_file=dict(extensions=None, ),
        syn_file=dict(extensions=None, ),
    )
    outputs = FitAsl.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..confounds import FramewiseDisplacement


def test_FramewiseDisplacement_inputs():
    input_map = dict(
        figdpi=dict(usedefault=True, ),
        figsize=dict(usedefault=True, ),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        normalize=dict(usedefault=True, ),
        out_figure=dict(
            extensions=None,
            usedefault=True,
        ),
        out_file=dict(
            extensions=None,
            usedefault=True,
        ),
        parameter_source=dict(mandatory=True, ),
        radius=dict(usedefault=True, ),
        save_plot=dict(usedefault=True, ),
        series_tr=dict(),
    )
    inputs = FramewiseDisplacement.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FramewiseDisplacement_outputs():
    output_map = dict(
        fd_average=dict(),
        out_figure=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
    )
    outputs = FramewiseDisplacement.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brains import ImageRegionPlotter


def test_ImageRegionPlotter_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputBinaryROIVolume=dict(
            argstr='--inputBinaryROIVolume %s',
            extensions=None,
        ),
        inputLabelVolume=dict(
            argstr='--inputLabelVolume %s',
            extensions=None,
        ),
        inputVolume1=dict(
            argstr='--inputVolume1 %s',
            extensions=None,
        ),
        inputVolume2=dict(
            argstr='--inputVolume2 %s',
            extensions=None,
        ),
        numberOfHistogramBins=dict(argstr='--numberOfHistogramBins %d', ),
        outputJointHistogramData=dict(
            argstr='--outputJointHistogramData %s', ),
        useIntensityForHistogram=dict(argstr='--useIntensityForHistogram ', ),
        useROIAUTO=dict(argstr='--useROIAUTO ', ),
        verbose=dict(argstr='--verbose ', ),
    )
    inputs = ImageRegionPlotter.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ImageRegionPlotter_outputs():
    output_map = dict()
    outputs = ImageRegionPlotter.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .diffusion import dtiaverage, dtiestim, dtiprocess, DWIConvert
from .tractography import *
from .gtract import (
    gtractTransformToDisplacementField, gtractInvertBSplineTransform,
    gtractConcatDwi, gtractAverageBvalues, gtractCoregBvalues,
    gtractResampleAnisotropy, gtractResampleCodeImage,
    gtractCopyImageOrientation, gtractCreateGuideFiber, gtractAnisotropyMap,
    gtractClipAnisotropy, gtractResampleB0, gtractInvertRigidTransform,
    gtractImageConformity, compareTractInclusion, gtractFastMarchingTracking,
    gtractInvertDisplacementField, gtractCoRegAnatomy,
    gtractResampleDWIInPlace, gtractCostFastMarching, gtractFiberTracking,
    extractNrrdVectorIndex, gtractResampleFibers, gtractTensor)
from .maxcurvature import maxcurvature



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..io import FreeSurferSource


def test_FreeSurferSource_inputs():
    input_map = dict(
        hemi=dict(usedefault=True, ),
        subject_id=dict(mandatory=True, ),
        subjects_dir=dict(mandatory=True, ),
    )
    inputs = FreeSurferSource.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FreeSurferSource_outputs():
    output_map = dict(
        BA_stats=dict(
            altkey='BA',
            loc='stats',
        ),
        T1=dict(
            extensions=None,
            loc='mri',
        ),
        annot=dict(
            altkey='*annot',
            loc='label',
        ),
        aparc_a2009s_stats=dict(
            altkey='aparc.a2009s',
            loc='stats',
        ),
        aparc_aseg=dict(
            altkey='aparc*aseg',
            loc='mri',
        ),
        aparc_stats=dict(
            altkey='aparc',
            loc='stats',
        ),
        area_pial=dict(
            altkey='area.pial',
            loc='surf',
        ),
        aseg=dict(
            extensions=None,
            loc='mri',
        ),
        aseg_stats=dict(
            altkey='aseg',
            loc='stats',
        ),
        avg_curv=dict(loc='surf', ),
        brain=dict(
            extensions=None,
            loc='mri',
        ),
        brainmask=dict(
            extensions=None,
            loc='mri',
        ),
        curv=dict(loc='surf', ),
        curv_pial=dict(
            altkey='curv.pial',
            loc='surf',
        ),
        curv_stats=dict(
            altkey='curv',
            loc='stats',
        ),
        entorhinal_exvivo_stats=dict(
            altkey='entorhinal_exvivo',
            loc='stats',
        ),
        filled=dict(
            extensions=None,
            loc='mri',
        ),
        graymid=dict(
            altkey=['graymid', 'midthickness'],
            loc='surf',
        ),
        inflated=dict(loc='surf', ),
        jacobian_white=dict(loc='surf', ),
        label=dict(
            altkey='*label',
            loc='label',
        ),
        norm=dict(
            extensions=None,
            loc='mri',
        ),
        nu=dict(
            extensions=None,
            loc='mri',
        ),
        orig=dict(
            extensions=None,
            loc='mri',
        ),
        pial=dict(loc='surf', ),
        rawavg=dict(
            extensions=None,
            loc='mri',
        ),
        ribbon=dict(
            altkey='*ribbon',
            loc='mri',
        ),
        smoothwm=dict(loc='surf', ),
        sphere=dict(loc='surf', ),
        sphere_reg=dict(
            altkey='sphere.reg',
            loc='surf',
        ),
        sulc=dict(loc='surf', ),
        thickness=dict(loc='surf', ),
        volume=dict(loc='surf', ),
        white=dict(loc='surf', ),
        wm=dict(
            extensions=None,
            loc='mri',
        ),
        wmparc=dict(
            extensions=None,
            loc='mri',
        ),
        wmparc_stats=dict(
            altkey='wmparc',
            loc='stats',
        ),
    )
    outputs = FreeSurferSource.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class ExtractSkeletonInputSpec(CommandLineInputSpec):
    InputImageFileName = File(
        position=-2, desc="Input image", exists=True, argstr="%s")
    OutputImageFileName = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Skeleton of the input image",
        argstr="%s")
    type = traits.Enum(
        "1D", "2D", desc="Type of skeleton to create", argstr="--type %s")
    dontPrune = traits.Bool(
        desc="Return the full skeleton, not just the maximal skeleton",
        argstr="--dontPrune ")
    numPoints = traits.Int(
        desc="Number of points used to represent the skeleton",
        argstr="--numPoints %d")
    pointsFile = traits.Str(
        desc=
        "Name of the file to store the coordinates of the central (1D) skeleton points",
        argstr="--pointsFile %s")


class ExtractSkeletonOutputSpec(TraitedSpec):
    OutputImageFileName = File(
        position=-1, desc="Skeleton of the input image", exists=True)


class ExtractSkeleton(SEMLikeCommandLine):
    """title: Extract Skeleton

category: Filtering

description: Extract the skeleton of a binary object.  The skeleton can be limited to being a 1D curve or allowed to be a full 2D manifold.  The branches of the skeleton can be pruned so that only the maximal center skeleton is returned.

version: 0.1.0.$Revision: 2104 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/ExtractSkeleton

contributor: Pierre Seroul (UNC), Martin Styner (UNC), Guido Gerig (UNC), Stephen Aylward (Kitware)

acknowledgements: The original implementation of this method was provided by ETH Zurich, Image Analysis Laboratory of Profs Olaf Kuebler, Gabor Szekely and Guido Gerig.  Martin Styner at UNC, Chapel Hill made enhancements.  Wrapping for Slicer was provided by Pierre Seroul and Stephen Aylward at Kitware, Inc.

"""

    input_spec = ExtractSkeletonInputSpec
    output_spec = ExtractSkeletonOutputSpec
    _cmd = "ExtractSkeleton "
    _outputs_filenames = {'OutputImageFileName': 'OutputImageFileName.nii'}



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import FNIRT


def test_FNIRT_inputs():
    input_map = dict(
        affine_file=dict(
            argstr='--aff=%s',
            extensions=None,
        ),
        apply_inmask=dict(
            argstr='--applyinmask=%s',
            sep=',',
            xor=['skip_inmask'],
        ),
        apply_intensity_mapping=dict(
            argstr='--estint=%s',
            sep=',',
            xor=['skip_intensity_mapping'],
        ),
        apply_refmask=dict(
            argstr='--applyrefmask=%s',
            sep=',',
            xor=['skip_refmask'],
        ),
        args=dict(argstr='%s', ),
        bias_regularization_lambda=dict(argstr='--biaslambda=%f', ),
        biasfield_resolution=dict(argstr='--biasres=%d,%d,%d', ),
        config_file=dict(argstr='--config=%s', ),
        derive_from_ref=dict(argstr='--refderiv', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        field_file=dict(
            argstr='--fout=%s',
            hash_files=False,
        ),
        fieldcoeff_file=dict(argstr='--cout=%s', ),
        hessian_precision=dict(argstr='--numprec=%s', ),
        in_file=dict(
            argstr='--in=%s',
            extensions=None,
            mandatory=True,
        ),
        in_fwhm=dict(
            argstr='--infwhm=%s',
            sep=',',
        ),
        in_intensitymap_file=dict(
            argstr='--intin=%s',
            copyfile=False,
        ),
        inmask_file=dict(
            argstr='--inmask=%s',
            extensions=None,
        ),
        inmask_val=dict(argstr='--impinval=%f', ),
        intensity_mapping_model=dict(argstr='--intmod=%s', ),
        intensity_mapping_order=dict(argstr='--intorder=%d', ),
        inwarp_file=dict(
            argstr='--inwarp=%s',
            extensions=None,
        ),
        jacobian_file=dict(
            argstr='--jout=%s',
            hash_files=False,
        ),
        jacobian_range=dict(argstr='--jacrange=%f,%f', ),
        log_file=dict(
            argstr='--logout=%s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        max_nonlin_iter=dict(
            argstr='--miter=%s',
            sep=',',
        ),
        modulatedref_file=dict(
            argstr='--refout=%s',
            hash_files=False,
        ),
        out_intensitymap_file=dict(
            argstr='--intout=%s',
            hash_files=False,
        ),
        output_type=dict(),
        ref_file=dict(
            argstr='--ref=%s',
            extensions=None,
            mandatory=True,
        ),
        ref_fwhm=dict(
            argstr='--reffwhm=%s',
            sep=',',
        ),
        refmask_file=dict(
            argstr='--refmask=%s',
            extensions=None,
        ),
        refmask_val=dict(argstr='--imprefval=%f', ),
        regularization_lambda=dict(
            argstr='--lambda=%s',
            sep=',',
        ),
        regularization_model=dict(argstr='--regmod=%s', ),
        skip_implicit_in_masking=dict(argstr='--impinm=0', ),
        skip_implicit_ref_masking=dict(argstr='--imprefm=0', ),
        skip_inmask=dict(
            argstr='--applyinmask=0',
            xor=['apply_inmask'],
        ),
        skip_intensity_mapping=dict(
            argstr='--estint=0',
            xor=['apply_intensity_mapping'],
        ),
        skip_lambda_ssq=dict(argstr='--ssqlambda=0', ),
        skip_refmask=dict(
            argstr='--applyrefmask=0',
            xor=['apply_refmask'],
        ),
        spline_order=dict(argstr='--splineorder=%d', ),
        subsampling_scheme=dict(
            argstr='--subsamp=%s',
            sep=',',
        ),
        warp_resolution=dict(argstr='--warpres=%d,%d,%d', ),
        warped_file=dict(
            argstr='--iout=%s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
    )
    inputs = FNIRT.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FNIRT_outputs():
    output_map = dict(
        field_file=dict(extensions=None, ),
        fieldcoeff_file=dict(extensions=None, ),
        jacobian_file=dict(extensions=None, ),
        log_file=dict(extensions=None, ),
        modulatedref_file=dict(extensions=None, ),
        out_intensitymap_file=dict(),
        warped_file=dict(extensions=None, ),
    )
    outputs = FNIRT.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..gtract import gtractCostFastMarching


def test_gtractCostFastMarching_inputs():
    input_map = dict(
        anisotropyWeight=dict(argstr='--anisotropyWeight %f', ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputAnisotropyVolume=dict(
            argstr='--inputAnisotropyVolume %s',
            extensions=None,
        ),
        inputStartingSeedsLabelMapVolume=dict(
            argstr='--inputStartingSeedsLabelMapVolume %s',
            extensions=None,
        ),
        inputTensorVolume=dict(
            argstr='--inputTensorVolume %s',
            extensions=None,
        ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        outputCostVolume=dict(
            argstr='--outputCostVolume %s',
            hash_files=False,
        ),
        outputSpeedVolume=dict(
            argstr='--outputSpeedVolume %s',
            hash_files=False,
        ),
        seedThreshold=dict(argstr='--seedThreshold %f', ),
        startingSeedsLabel=dict(argstr='--startingSeedsLabel %d', ),
        stoppingValue=dict(argstr='--stoppingValue %f', ),
    )
    inputs = gtractCostFastMarching.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_gtractCostFastMarching_outputs():
    output_map = dict(
        outputCostVolume=dict(extensions=None, ),
        outputSpeedVolume=dict(extensions=None, ),
    )
    outputs = gtractCostFastMarching.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..tensors import ConstrainedSphericalDeconvolution


def test_ConstrainedSphericalDeconvolution_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        debug=dict(argstr='-debug', ),
        directions_file=dict(
            argstr='-directions %s',
            extensions=None,
            position=-2,
        ),
        encoding_file=dict(
            argstr='-grad %s',
            extensions=None,
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filter_file=dict(
            argstr='-filter %s',
            extensions=None,
            position=-2,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        iterations=dict(argstr='-niter %s', ),
        lambda_value=dict(argstr='-lambda %s', ),
        mask_image=dict(
            argstr='-mask %s',
            extensions=None,
            position=2,
        ),
        maximum_harmonic_order=dict(argstr='-lmax %s', ),
        normalise=dict(
            argstr='-normalise',
            position=3,
        ),
        out_filename=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        response_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        threshold_value=dict(argstr='-threshold %s', ),
    )
    inputs = ConstrainedSphericalDeconvolution.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ConstrainedSphericalDeconvolution_outputs():
    output_map = dict(spherical_harmonics_image=dict(extensions=None, ), )
    outputs = ConstrainedSphericalDeconvolution.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ImageMeants


def test_ImageMeants_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        eig=dict(argstr='--eig', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
        ),
        nobin=dict(argstr='--no_bin', ),
        order=dict(
            argstr='--order=%d',
            usedefault=True,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_type=dict(),
        show_all=dict(argstr='--showall', ),
        spatial_coord=dict(argstr='-c %s', ),
        transpose=dict(argstr='--transpose', ),
        use_mm=dict(argstr='--usemm', ),
    )
    inputs = ImageMeants.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ImageMeants_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = ImageMeants.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..convert import AnalyzeHeader


def test_AnalyzeHeader_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        centre=dict(
            argstr='-centre %s',
            units='mm',
        ),
        data_dims=dict(
            argstr='-datadims %s',
            units='voxels',
        ),
        datatype=dict(
            argstr='-datatype %s',
            mandatory=True,
        ),
        description=dict(argstr='-description %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        greylevels=dict(
            argstr='-gl %s',
            units='NA',
        ),
        in_file=dict(
            argstr='< %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        initfromheader=dict(
            argstr='-initfromheader %s',
            extensions=None,
            position=3,
        ),
        intelbyteorder=dict(argstr='-intelbyteorder', ),
        networkbyteorder=dict(argstr='-networkbyteorder', ),
        nimages=dict(
            argstr='-nimages %d',
            units='NA',
        ),
        offset=dict(
            argstr='-offset %d',
            units='NA',
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        picoseed=dict(
            argstr='-picoseed %s',
            units='mm',
        ),
        printbigendian=dict(
            argstr='-printbigendian %s',
            extensions=None,
            position=3,
        ),
        printimagedims=dict(
            argstr='-printimagedims %s',
            extensions=None,
            position=3,
        ),
        printintelbyteorder=dict(
            argstr='-printintelbyteorder %s',
            extensions=None,
            position=3,
        ),
        printprogargs=dict(
            argstr='-printprogargs %s',
            extensions=None,
            position=3,
        ),
        readheader=dict(
            argstr='-readheader %s',
            extensions=None,
            position=3,
        ),
        scaleinter=dict(
            argstr='-scaleinter %d',
            units='NA',
        ),
        scaleslope=dict(
            argstr='-scaleslope %d',
            units='NA',
        ),
        scheme_file=dict(
            argstr='%s',
            extensions=None,
            position=2,
        ),
        voxel_dims=dict(
            argstr='-voxeldims %s',
            units='mm',
        ),
    )
    inputs = AnalyzeHeader.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_AnalyzeHeader_outputs():
    output_map = dict(header=dict(extensions=None, ), )
    outputs = AnalyzeHeader.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""Parallel workflow execution via SGE
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open

import os
import sys

from ...interfaces.base import CommandLine
from .base import (GraphPluginBase, logger)


def node_completed_status(checknode):
    """
    A function to determine if a node has previously completed it's work
    :param checknode: The node to check the run status
    :return: boolean value True indicates that the node does not need to be run.
    """
    """ TODO: place this in the base.py file and refactor """
    node_state_does_not_require_overwrite = (
        checknode.overwrite is False or
        (checknode.overwrite is None and not checknode._interface.always_run))
    hash_exists = False
    try:
        hash_exists, _, _, _ = checknode.hash_exists()
    except Exception:
        hash_exists = False
    return (hash_exists and node_state_does_not_require_overwrite)


class SGEGraphPlugin(GraphPluginBase):
    """Execute using SGE

    The plugin_args input to run can be used to control the SGE execution.
    Currently supported options are:

    - template : template to use for batch job submission
    - qsub_args : arguments to be prepended to the job execution script in the
                  qsub call

    """
    _template = """
#!/bin/bash
#$ -V
#$ -S /bin/bash
"""

    def __init__(self, **kwargs):
        self._qsub_args = ''
        self._dont_resubmit_completed_jobs = False
        if 'plugin_args' in kwargs and kwargs['plugin_args']:
            plugin_args = kwargs['plugin_args']
            if 'template' in plugin_args:
                self._template = plugin_args['template']
                if os.path.isfile(self._template):
                    self._template = open(self._template).read()
            if 'qsub_args' in plugin_args:
                self._qsub_args = plugin_args['qsub_args']
            if 'dont_resubmit_completed_jobs' in plugin_args:
                self._dont_resubmit_completed_jobs = plugin_args[
                    'dont_resubmit_completed_jobs']
        super(SGEGraphPlugin, self).__init__(**kwargs)

    def _submit_graph(self, pyfiles, dependencies, nodes):
        def make_job_name(jobnumber, nodeslist):
            """
            - jobnumber: The index number of the job to create
            - nodeslist: The name of the node being processed
            - return: A string representing this job to be displayed by SGE
            """
            job_name = 'j{0}_{1}'.format(jobnumber, nodeslist[jobnumber]._id)
            # Condition job_name to be a valid bash identifier (i.e. - is invalid)
            job_name = job_name.replace('-', '_').replace('.', '_').replace(
                ':', '_')
            return job_name

        batch_dir, _ = os.path.split(pyfiles[0])
        submitjobsfile = os.path.join(batch_dir, 'submit_jobs.sh')

        cache_doneness_per_node = dict()
        if self._dont_resubmit_completed_jobs:  # A future parameter for controlling this behavior could be added here
            for idx, pyscript in enumerate(pyfiles):
                node = nodes[idx]
                node_status_done = node_completed_status(node)

                # if the node itself claims done, then check to ensure all
                # dependancies are also done
                if node_status_done and idx in dependencies:
                    for child_idx in dependencies[idx]:
                        if child_idx in cache_doneness_per_node:
                            child_status_done = cache_doneness_per_node[
                                child_idx]
                        else:
                            child_status_done = node_completed_status(
                                nodes[child_idx])
                        node_status_done = node_status_done and child_status_done

                cache_doneness_per_node[idx] = node_status_done

        with open(submitjobsfile, 'wt') as fp:
            fp.writelines('#!/usr/bin/env bash\n')
            fp.writelines('# Condense format attempted\n')
            for idx, pyscript in enumerate(pyfiles):
                node = nodes[idx]
                if cache_doneness_per_node.get(idx, False):
                    continue
                else:
                    template, qsub_args = self._get_args(
                        node, ["template", "qsub_args"])

                    batch_dir, name = os.path.split(pyscript)
                    name = '.'.join(name.split('.')[:-1])
                    batchscript = '\n'.join(
                        (template, '%s %s' % (sys.executable, pyscript)))
                    batchscriptfile = os.path.join(batch_dir,
                                                   'batchscript_%s.sh' % name)

                    batchscriptoutfile = batchscriptfile + '.o'
                    batchscripterrfile = batchscriptfile + '.e'

                    with open(batchscriptfile, 'wt') as batchfp:
                        batchfp.writelines(batchscript)
                        batchfp.close()
                    deps = ''
                    if idx in dependencies:
                        values = ' '
                        for jobid in dependencies[idx]:
                            # Avoid dependancies of done jobs
                            if not self._dont_resubmit_completed_jobs or not cache_doneness_per_node[jobid]:
                                values += "${{{0}}},".format(
                                    make_job_name(jobid, nodes))
                        if values != ' ':  # i.e. if some jobs were added to dependency list
                            values = values.rstrip(',')
                            deps = '-hold_jid%s' % values
                    jobname = make_job_name(idx, nodes)
                    # Do not use default output locations if they are set in self._qsub_args
                    stderrFile = ''
                    if self._qsub_args.count('-e ') == 0:
                        stderrFile = '-e {errFile}'.format(
                            errFile=batchscripterrfile)
                    stdoutFile = ''
                    if self._qsub_args.count('-o ') == 0:
                        stdoutFile = '-o {outFile}'.format(
                            outFile=batchscriptoutfile)
                    full_line = '{jobNm}=$(qsub {outFileOption} {errFileOption} {extraQSubArgs} {dependantIndex} -N {jobNm} {batchscript} | awk \'{{print $3}}\')\n'.format(
                        jobNm=jobname,
                        outFileOption=stdoutFile,
                        errFileOption=stderrFile,
                        extraQSubArgs=qsub_args,
                        dependantIndex=deps,
                        batchscript=batchscriptfile)
                    fp.writelines(full_line)
        cmd = CommandLine(
            'bash',
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output='allatonce')
        cmd.inputs.args = '%s' % submitjobsfile
        cmd.run()
        logger.info('submitted all jobs to queue')



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from ... import logging, LooseVersion
from ...utils.filemanip import which
from ..base import (CommandLineInputSpec, CommandLine, traits, File, isdefined, PackageInfo)
iflogger = logging.getLogger('nipype.interface')


class Info(PackageInfo):
    version_cmd = 'mrconvert --version'

    @staticmethod
    def parse_version(raw_info):
        # info is like: "== mrconvert 0.3.15-githash"
        for line in raw_info.splitlines():
            if line.startswith('== mrconvert '):
                v_string = line.split()[2]
                break
        else:
            return None

        # -githash may or may not be appended
        v_string = v_string.split('-')[0]

        return '.'.join(v_string.split('.')[:3])

    @classmethod
    def looseversion(cls):
        """ Return a comparable version object

        If no version found, use LooseVersion('0.0.0')
        """
        return LooseVersion(cls.version() or '0.0.0')


class MRTrix3BaseInputSpec(CommandLineInputSpec):
    nthreads = traits.Int(
        argstr='-nthreads %d',
        desc='number of threads. if zero, the number'
        ' of available cpus will be used',
        nohash=True)
    # DW gradient table import options
    grad_file = File(
        exists=True,
        argstr='-grad %s',
        desc='dw gradient scheme (MRTrix format)',
        xor=['grad_fsl'])
    grad_fsl = traits.Tuple(
        File(exists=True),
        File(exists=True),
        argstr='-fslgrad %s %s',
        desc='(bvecs, bvals) dw gradient scheme (FSL format)',
        xor=['grad_file'])
    bval_scale = traits.Enum(
        'yes',
        'no',
        argstr='-bvalue_scaling %s',
        desc='specifies whether the b - values should be scaled by the square'
        ' of the corresponding DW gradient norm, as often required for '
        'multishell or DSI DW acquisition schemes. The default action '
        'can also be set in the MRtrix config file, under the '
        'BValueScaling entry. Valid choices are yes / no, true / '
        'false, 0 / 1 (default: true).')

    in_bvec = File(
        exists=True, argstr='-fslgrad %s %s', desc='bvecs file in FSL format')
    in_bval = File(exists=True, desc='bvals file in FSL format')


class MRTrix3Base(CommandLine):
    def _format_arg(self, name, trait_spec, value):
        if name == 'nthreads' and value == 0:
            value = 1
            try:
                from multiprocessing import cpu_count
                value = cpu_count()
            except:
                iflogger.warning('Number of threads could not be computed')
                pass
            return trait_spec.argstr % value

        if name == 'in_bvec':
            return trait_spec.argstr % (value, self.inputs.in_bval)

        return super(MRTrix3Base, self)._format_arg(name, trait_spec, value)

    def _parse_inputs(self, skip=None):
        if skip is None:
            skip = []

        try:
            if (isdefined(self.inputs.grad_file)
                    or isdefined(self.inputs.grad_fsl)):
                skip += ['in_bvec', 'in_bval']

            is_bvec = isdefined(self.inputs.in_bvec)
            is_bval = isdefined(self.inputs.in_bval)
            if is_bvec or is_bval:
                if not is_bvec or not is_bval:
                    raise RuntimeError('If using bvecs and bvals inputs, both'
                                       'should be defined')
                skip += ['in_bval']
        except AttributeError:
            pass

        return super(MRTrix3Base, self)._parse_inputs(skip=skip)

    @property
    def version(self):
        return Info.version()



"""Test afni deconvolve"""

from ..model import Deconvolve

def test_x1dstop():
    deconv = Deconvolve()
    deconv.inputs.out_file = 'file.nii'
    assert 'out_file' in deconv._list_outputs()
    deconv.inputs.x1D_stop = True
    assert 'out_file' not in deconv._list_outputs()
    assert 'cbucket' not in deconv._list_outputs()



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import XFibres5


def test_XFibres5_inputs():
    input_map = dict(
        all_ard=dict(
            argstr='--allard',
            xor=('no_ard', 'all_ard'),
        ),
        args=dict(argstr='%s', ),
        burn_in=dict(
            argstr='--burnin=%d',
            usedefault=True,
        ),
        burn_in_no_ard=dict(
            argstr='--burnin_noard=%d',
            usedefault=True,
        ),
        bvals=dict(
            argstr='--bvals=%s',
            extensions=None,
            mandatory=True,
        ),
        bvecs=dict(
            argstr='--bvecs=%s',
            extensions=None,
            mandatory=True,
        ),
        cnlinear=dict(
            argstr='--cnonlinear',
            xor=('no_spat', 'non_linear', 'cnlinear'),
        ),
        dwi=dict(
            argstr='--data=%s',
            extensions=None,
            mandatory=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        f0_ard=dict(
            argstr='--f0 --ardf0',
            xor=['f0_noard', 'f0_ard', 'all_ard'],
        ),
        f0_noard=dict(
            argstr='--f0',
            xor=['f0_noard', 'f0_ard'],
        ),
        force_dir=dict(
            argstr='--forcedir',
            usedefault=True,
        ),
        fudge=dict(argstr='--fudge=%d', ),
        gradnonlin=dict(
            argstr='--gradnonlin=%s',
            extensions=None,
        ),
        logdir=dict(
            argstr='--logdir=%s',
            usedefault=True,
        ),
        mask=dict(
            argstr='--mask=%s',
            extensions=None,
            mandatory=True,
        ),
        model=dict(argstr='--model=%d', ),
        n_fibres=dict(
            argstr='--nfibres=%d',
            mandatory=True,
            usedefault=True,
        ),
        n_jumps=dict(
            argstr='--njumps=%d',
            usedefault=True,
        ),
        no_ard=dict(
            argstr='--noard',
            xor=('no_ard', 'all_ard'),
        ),
        no_spat=dict(
            argstr='--nospat',
            xor=('no_spat', 'non_linear', 'cnlinear'),
        ),
        non_linear=dict(
            argstr='--nonlinear',
            xor=('no_spat', 'non_linear', 'cnlinear'),
        ),
        output_type=dict(),
        rician=dict(argstr='--rician', ),
        sample_every=dict(
            argstr='--sampleevery=%d',
            usedefault=True,
        ),
        seed=dict(argstr='--seed=%d', ),
        update_proposal_every=dict(
            argstr='--updateproposalevery=%d',
            usedefault=True,
        ),
    )
    inputs = XFibres5.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_XFibres5_outputs():
    output_map = dict(
        dyads=dict(),
        fsamples=dict(),
        mean_S0samples=dict(extensions=None, ),
        mean_dsamples=dict(extensions=None, ),
        mean_fsamples=dict(),
        mean_tausamples=dict(extensions=None, ),
        phsamples=dict(),
        thsamples=dict(),
    )
    outputs = XFibres5.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
